
' .ApplicationSource CRV-Basics(1.0)'!

'From VisualWorks(R), Release 2.5 of September 26, 1995 on August 24, 1997 at 6:17:53 pm'!

'- user-strings of CRV-Basics(1.0) -'!

#('name' 'CRV-Basics' 'version' '1.0')!

'required applications for CRV-Basics(1.0)'!

Message
	selector: #checkRequired:
	argument: #()!

'- source of CRV-Basics(1.0) -'!



Point subclass: #ControlPoint
	instanceVariableNames: 'weight pressure saturation '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CurveBasics'!
ControlPoint comment:
'I am a control point.

Instance Variables:
	pressure	Float	current pressure
	saturation	Float	current saturation
	weight		Float	current weight
'!

!ControlPoint methodsFor: 'accessing'!

coordinates

	^x@y!

coordinates: aPoint

	x := aPoint x.
	y := aPoint y.!

coordinates: aPoint weight: aNumber1 pressure: aNumber2 saturation: aNumber3

		self coordinates: aPoint.
		weight := aNumber1.
		pressure := aNumber2.
		saturation := aNumber3.!

pressure

	^pressure!

pressure: aNumber

	pressure := aNumber!

saturation

	^saturation!

saturation: aNumber

	saturation := aNumber!

weight

	^weight!

weight: aNumber

	weight := aNumber! !

!ControlPoint methodsFor: 'converting'!

asPoint

	^x@y!

blendWith: aControlPoint value: aNumber

	"blend self with aControlPoint, 0 means self and 1 means aControlPoint"

	^self class
		coordinates: (self coordinates blendWith: aControlPoint coordinates value: aNumber)
		weight: (self weight blendWith: aControlPoint weight value: aNumber)
		pressure: (self pressure blendWith: aControlPoint pressure value: aNumber)
		saturation: (self saturation blendWith: aControlPoint saturation value: aNumber)!

width

	^pressure!

width: aNumber

	pressure := aNumber! !

!ControlPoint methodsFor: 'comparing'!

= anObject 

	^super = anObject and:
		[weight = anObject weight and:
			[pressure = anObject pressure and:
				[saturation = anObject saturation]]]!

hash

	^(x + y + weight + pressure + saturation) asFloat floatHash! !

!ControlPoint methodsFor: 'transforming'!

scaleBy: amount 
	"Answer the receiver scaled by the argument amount, which can be
	a Point or a scalar value."

	^self coordinates: (self coordinates scaledBy: amount)!

scaledBy: amount 
	"Answer a new object scaled by the argument amount, which can be
	a Point or a scalar value."

	^self copy scaleBy: amount!

translateBy: amount 
	"Answer the receiver translated by amount, which can be
	a Point or a scalar value."

	^self coordinates: (self coordinates translatedBy: amount)!

translatedBy: amount 
	"Answer a new object translated by amount, which can be
	a Point or a scalar value."

	^self copy translateBy: amount! !

!ControlPoint methodsFor: 'point functions'!

variate: r

	^self copy coordinates: (self coordinates variate: r)! !

!ControlPoint methodsFor: 'printing'!

printOn: aStream 

	aStream print: self class; space.
	self saveTxtOn: aStream!

storeOn: aStream 

	aStream
		store: self class;
		nextPutAll: ' coordinates: '; print: self coordinates;
		nextPutAll: 'weight: '; print: weight;
		nextPutAll: 'pressure: '; print: pressure;
		nextPutAll: 'saturation: '; print: saturation! !

!ControlPoint methodsFor: 'save txt / print eps'!

saveTxtOn: aStream 
	"save self in a special TXT-format"

	aStream
		nextPutAll: 'x: '; print: x; tab;
		nextPutAll: 'y: '; print: y.
	weight = 1.0 ifFalse: [aStream tab; nextPutAll: 'w: '; print: weight].
	pressure = 1.0 ifFalse: [aStream tab; nextPutAll: 'p: '; print: pressure].
	saturation = 1.0 ifFalse: [aStream tab; nextPutAll: 's: '; print: saturation].! !

!ControlPoint methodsFor: 'edit'!

edit

	| dialog builder doEdit x1 y1 w1 p1 s1 |

	dialog := SimpleDialog new.
	builder := dialog builder.

	builder aspectAt: #x put: (x1 := x asValue).
	builder aspectAt: #y put: (y1 := y asValue).
	builder aspectAt: #w put: (w1 := weight asValue).

	builder aspectAt: #p put: (p1 := pressure asValue).
	builder aspectAt: #vp put: (VisualBlock block:
		[ :gc :bounds | | rect |
			rect := bounds insetBy: 1.
			gc
				paint: ColorValue black; lineWidth: p1 value;
				displayLineFrom: (rect leftCenter) to: (rect rightCenter).
		]).

	builder aspectAt: #s put: (s1 := saturation asValue).
	builder aspectAt: #vs put: (VisualBlock block:
		[ :gc :bounds |
			gc
				paint: (ColorValue brightness: 1.0 - s1 value);
				displayRectangle: (bounds insetBy: 1).
		]).

	builder aspectAt: #doAccept put: [doEdit := true. dialog close].
	builder aspectAt: #doCancel put: [doEdit := false. dialog close].

	dialog postBuildBlock: [ :app :aBuilder | 
		(aBuilder aspectAt: #p) onChangeSend: #invalidate to: (aBuilder componentAt: #vp).
		(aBuilder aspectAt: #s) onChangeSend: #invalidate to: (aBuilder componentAt: #vs).
	].

	dialog openFrom: (UISpecification from: self class editDialogSpec).

	doEdit ifTrue: [
		x := x1 value.
		y := y1 value.
		weight := w1 value.
		pressure := p1 value.
		pressure isInteger ifFalse: [pressure := pressure asFloat].
		saturation := s1 value.
		saturation isInteger ifFalse: [saturation := saturation asFloat].
	]! !

!ControlPoint class methodsFor: 'instance creation'!

coordinates: aPoint

	^self
		coordinates: aPoint
		weight: 1.0!

coordinates: aPoint weight: aNumber

	^self 
		coordinates: aPoint
		weight: aNumber
		pressure: 1.0!

coordinates: aPoint weight: aNumber1 pressure: aNumber2

	^self 
		coordinates: aPoint
		weight: aNumber1
		pressure: aNumber2
		saturation: 1.0!

coordinates: aPoint weight: aNumber1 pressure: aNumber2 saturation: aNumber3

	^self basicNew
		coordinates: aPoint
		weight: aNumber1
		pressure: aNumber2
		saturation: aNumber3! !

!ControlPoint class methodsFor: 'interface specs'!

editDialogSpec
	"UIPainter new openOnClass: self andSelector: #editDialogSpec"

	<resource: #canvas>
	^#(#FullSpec 
		#window: 
		#(#WindowSpec 
			#label: 'ControlPoint' 
			#min: #(#Point 313 167 ) 
			#max: #(#Point 313 167 ) 
			#bounds: #(#Rectangle 180 154 493 321 ) ) 
		#component: 
		#(#SpecCollection 
			#collection: #(
				#(#SliderSpec 
					#layout: #(#Rectangle 168 32 256 48 ) 
					#name: #p 
					#colors: 
					#(#LookPreferences 
						#setSelectionBackgroundColor: #(#ColorValue 6553 6553 6553 ) ) 
					#model: #p 
					#orientation: #horizontal 
					#start: 1.0 
					#stop: 26.0 ) 
				#(#SliderSpec 
					#layout: #(#Rectangle 168 88 256 104 ) 
					#name: #s 
					#colors: 
					#(#LookPreferences 
						#setSelectionBackgroundColor: #(#ColorValue 6553 6553 6553 ) ) 
					#model: #s 
					#orientation: #horizontal 
					#start: 0.0 
					#stop: 1.0 ) 
				#(#LabelSpec 
					#layout: #(#Point 168 8 ) 
					#label: 'pressure' ) 
				#(#LabelSpec 
					#layout: #(#Point 168 64 ) 
					#label: 'saturation' ) 
				#(#InputFieldSpec 
					#layout: #(#Rectangle 72 16 144 40 ) 
					#name: #x 
					#model: #x 
					#alignment: #right 
					#type: #number ) 
				#(#LabelSpec 
					#layout: #(#Point 16 16 ) 
					#label: 'x' ) 
				#(#InputFieldSpec 
					#layout: #(#Rectangle 72 48 144 72 ) 
					#name: #y 
					#model: #y 
					#alignment: #right 
					#type: #number ) 
				#(#LabelSpec 
					#layout: #(#Point 16 80 ) 
					#label: 'weight' ) 
				#(#InputFieldSpec 
					#layout: #(#Rectangle 72 80 144 104 ) 
					#name: #w 
					#model: #w 
					#alignment: #right 
					#type: #number ) 
				#(#LabelSpec 
					#layout: #(#Point 16 48 ) 
					#label: 'y' ) 
				#(#ArbitraryComponentSpec 
					#layout: #(#Rectangle 264 16 296 48 ) 
					#name: #vp 
					#component: #vp ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 48 128 128 160 ) 
					#model: #doAccept 
					#label: 'Apply' 
					#isDefault: true 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 184 128 264 160 ) 
					#model: #doCancel 
					#label: 'Cancel' 
					#defaultable: true ) 
				#(#ArbitraryComponentSpec 
					#layout: #(#Rectangle 264 72 296 104 ) 
					#name: #vs 
					#component: #vs ) 
				#(#GroupBoxSpec 
					#layout: #(#Rectangle 8 8 152 112 ) ) 
				#(#GroupBoxSpec 
					#layout: #(#Rectangle 160 64 304 112 ) ) 
				#(#GroupBoxSpec 
					#layout: #(#Rectangle 160 8 304 56 ) ) ) ) )! !

ControllerWithMenu subclass: #StyleController
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Style'!

!StyleController methodsFor: 'initialize-release'!

initialize

	| yellowButtonMenu |

	super initialize.

	yellowButtonMenu := self class yellowButtonMenu.
	(yellowButtonMenu menuItemLabeled: 'cyclic')
		indication: [self currentCurve cyclic].
	(yellowButtonMenu menuItemLabeled: 'control polygon')
		indication: [self view showControlPolygon].
	(yellowButtonMenu menuItemLabeled: 'control labels')
		indication: [self view showControlLabels].
	(yellowButtonMenu menuItemLabeled: 'grid')
		indication: [self view showGrid].

	self menuHolder: yellowButtonMenu asValue.! !

!StyleController methodsFor: 'control defaults'!

doubleClickAt: pt

	self view selectControlPointNear: pt.
	self view index notNil ifTrue: [
		self view invalidate.
		(self currentCurve controlPoints at: self view index) edit.
		self view unselectControlPoint.
		self currentCurve markDirty.
		self model changed.
	]!

redButtonActivity
	"ControlPointDrag"

	| pOld pNew |
	
	pOld := self sensor cursorPoint.
	self view selectControlPointNear: pOld.
	self view index = nil
		ifTrue: [
			self view selectCurveNear: pOld.
			[self poll. self sensor redButtonPressed] whileTrue.
		]
		ifFalse: [ "move cursor at least 2 pixel before activate dragmode"
			[self poll. self sensor redButtonPressed and:
				[(pOld dist: self sensor cursorPoint) < 2.0]] whileTrue.
			self sensor redButtonPressed ifTrue: [
				self view dragModeOn.
				self redButtonDragWhileTrue: [:p |
					pNew := InputState default ctrlDown
						ifTrue: [p roundTo: self view grid]
						ifFalse: [p].
					pOld ~= pNew ifTrue: [
						pOld := pNew.
						self view index notNil ifTrue: [ "avoid trouble with doubleClick"
							(self currentCurve controlPoints at: self view index) coordinates: pNew.
							InputState default shiftDown
								ifTrue: [self currentCurve markDirty. self model changed.]
								ifFalse: [self view invalidate]
						].
					].
				].
				self view unselectControlPoint.
				self view dragModeOff. "the model will changed"
				self currentCurve markDirty.
				self model changed.
			].
		].! !

!StyleController methodsFor: 'broadcast'!

downcastLocalEvent: aKey with: aParameter at: aPoint from: anInitiator 
	"This is a message passed down the view structure to some single 
	part. Answer true if we accepted the event, or false if it should be 
	passed on to whatever's behind us."

	aKey == #doubleClick ifTrue: [self doubleClickAt: aPoint. ^true].
	^super downcastLocalEvent: aKey with: aParameter at: aPoint from: anInitiator! !

!StyleController methodsFor: 'menu messages'!

cmDelete

	self model listOfCurves remove: self currentCurve.
	self view curveNr: 1.
	self model changed.!

cmFlipControlLabels

	self view showControlLabels: self view showControlLabels not!

cmFlipControlPolygon

	self view showControlPolygon: self view showControlPolygon not!

cmFlipCyclic

	self currentCurve cyclic: self currentCurve cyclic not.
	self model changed.!

cmFlipGrid

	self view showGrid: self view showGrid not!

cmFromUserAdd

	| crv |

	crv := self fromUser.
	crv notNil ifTrue: [self model listOfCurves add: crv. self model changed]!

cmFromUserReplace

	| crv |

	crv := self fromUser.
	crv notNil ifTrue: [
		self model listOfCurves
			at: (self model listOfCurves find: self currentCurve)
			put: crv.
		self model changed.
	]!

cmInspect

	self model inspect!

cmInspectCurrentCurve

	self currentCurve inspect!

cmInterpolationBSpline

	self currentCurve interpolation: #bSpline.
	self model changed.!

cmInterpolationCatmull

	self currentCurve interpolation: #catmull.
	self model changed.!

cmInterpolationChordal

	self currentCurve interpolation: #chordal.
	self model changed.!

cmInterpolationPolyline

	self currentCurve interpolation: #polyline.
	self model changed.!

cmLoad

	| curves |

	(curves := Style loadTxt) notNil ifTrue: [
		self model listOfCurves: curves listOfCurves.
		self view curveNr: 1.
		self model changed.
	].!

cmLoadBackground

	self view loadBackground!

cmPrintEps

	self view printEps!

cmReverse

	self currentCurve controlPoints: self currentCurve controlPoints reverse.
	self model changed.!

cmSave

	self model saveTxt!

cmSnapAllToGrid

	self currentCurve verticesDo: [ :cp |
		cp coordinates: (cp coordinates roundTo: self view grid)
	].
	self currentCurve markDirty.
	self model changed.!

cmStyleProperties

	self model editStyleProperties.
	self model changed.!

cmUpdate

	self model changed! !

!StyleController methodsFor: 'private'!

currentCurve

	^self model listOfCurves at: self view curveNr!

fromUser

	| gc sen points p |

	Cursor crossHair showWhile: [
		gc := self view graphicsContext.
		gc flush.
		sen := self sensor.
		points := OrderedCollection new.
		[	sen waitButton.
			p := sen mousePoint.
			InputState default ctrlDown ifTrue: [p := p roundTo: self view grid].
			sen redButtonPressed.] whileTrue: [
			sen waitNoButton.
			points add: (ControlPoint coordinates: p).
			self view drawMarkerOn: gc at: p label: points size.
			self checkForEvents.
		]
	].
	points size = 2 ifTrue: [^Curve controlPoints: points interpolation: #polyline].
	points size > 1 ifTrue: [^Curve controlPoints: points].
	^nil! !

!StyleController class methodsFor: 'resources'!

yellowButtonMenu
	"UIMenuEditor new openOnClass: self andSelector: #yellowButtonMenu"

	<resource: #menu>
	^#(#Menu #(
			#(#MenuItem 
				#rawLabel: 'interpolation' 
				#submenu: #(#Menu #(
						#(#MenuItem 
							#rawLabel: 'b-spline' 
							#value: #cmInterpolationBSpline ) 
						#(#MenuItem 
							#rawLabel: 'catmull' 
							#value: #cmInterpolationCatmull ) 
						#(#MenuItem 
							#rawLabel: 'polyline' 
							#value: #cmInterpolationPolyline ) 
						#(#MenuItem 
							#rawLabel: 'chordal' 
							#value: #cmInterpolationChordal ) ) #(4 ) nil ) ) 
			#(#MenuItem 
				#rawLabel: 'cyclic' 
				#value: #cmFlipCyclic ) 
			#(#MenuItem 
				#rawLabel: 'snap all to grid' 
				#submenu: #(#Menu #(
						#(#MenuItem 
							#rawLabel: 'confirm' 
							#value: #cmSnapAllToGrid ) ) #(1 ) nil ) ) 
			#(#MenuItem 
				#rawLabel: 'reverse' 
				#value: #cmReverse ) 
			#(#MenuItem 
				#rawLabel: 'add from user' 
				#value: #cmFromUserAdd ) 
			#(#MenuItem 
				#rawLabel: 'replace from user' 
				#value: #cmFromUserReplace ) 
			#(#MenuItem 
				#rawLabel: 'delete' 
				#submenu: #(#Menu #(
						#(#MenuItem 
							#rawLabel: 'confirm' 
							#value: #cmDelete ) ) #(1 ) nil ) ) 
			#(#MenuItem 
				#rawLabel: 'style properties...' 
				#value: #cmStyleProperties ) 
			#(#MenuItem 
				#rawLabel: 'load...' 
				#value: #cmLoad ) 
			#(#MenuItem 
				#rawLabel: 'save as...' 
				#value: #cmSave ) 
			#(#MenuItem 
				#rawLabel: 'print PS/EPS...' 
				#value: #cmPrintEps ) 
			#(#MenuItem 
				#rawLabel: 'control polygon' 
				#value: #cmFlipControlPolygon ) 
			#(#MenuItem 
				#rawLabel: 'control labels' 
				#value: #cmFlipControlLabels ) 
			#(#MenuItem 
				#rawLabel: 'grid' 
				#value: #cmFlipGrid ) 
			#(#MenuItem 
				#rawLabel: 'inspect' 
				#value: #cmInspect ) 
			#(#MenuItem 
				#rawLabel: 'inspect current curve' 
				#value: #cmInspectCurrentCurve ) 
			#(#MenuItem 
				#rawLabel: 'update' 
				#value: #cmUpdate ) ) #(7 4 3 3 ) nil ) decodeAsLiteralArray! !

ExternalLanguageParser subclass: #StyleCatalogParser
	instanceVariableNames: 'catalog package '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'StyleCatalog'!

!StyleCatalogParser methodsFor: 'initialize-release'!

initScanner

	super initScanner.
	catalog := Dictionary new.
	package := Dictionary new.! !

!StyleCatalogParser methodsFor: 'parsing'!

catalog =

	package+				[makeCatalog:]!

line =

	word $= string			[addLine:fileName:]!

package =

	$[ word	$]
	line+
							[addPackageName:package:]! !

!StyleCatalogParser methodsFor: 'private parsing support'!

addLine: s fileName: fileName

	package add: s asSymbol -> fileName!

addPackageName: packeageName package: d2

	catalog add: packeageName asSymbol -> package.
	package := Dictionary new.!

makeCatalog: d1

	^StyleCatalog catalog: catalog! !

!StyleCatalogParser class methodsFor: 'class initialization'!

initialize
	"StyleCatalogParser initialize"

	(self organization listAtCategoryNamed: #parsing) do: [ :selector |
		self recompile: selector
	].! !

!ColorValue methodsFor: 'save txt / print eps'!

saveTxtOn: aStream
	"save self in a special TXT-format"

	aStream nextPut: ${; print: self red; space; print: self green; space; print: self blue; nextPut: $}! !

View subclass: #StyleView
	instanceVariableNames: 'radius grid curveNr index dragMode showControlPolygon showControlLabels showGrid '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Style'!

!StyleView methodsFor: 'initialize-release'!

initialize

	super initialize.
	radius := 3.
	grid := 25.
	curveNr := 1.
	self unselectControlPoint.
	dragMode := false.
	showControlPolygon := true.
	showControlLabels := true.
	showGrid := true.! !

!StyleView methodsFor: 'accessing'!

curveNr

	^curveNr!

curveNr: aNumber

	(curveNr = aNumber or: [aNumber < 1 or: [aNumber > self model listOfCurves size]])
		ifFalse: [
			curveNr := aNumber.
			self invalidate.
		].!

grid

	^grid!

index

	^index!

showControlLabels

	^showControlLabels!

showControlLabels: aBoolean

	showControlLabels = aBoolean ifFalse: [
		showControlLabels := aBoolean.
		self invalidate.
	].!

showControlPolygon

	^showControlPolygon!

showControlPolygon: aBoolean

	showControlPolygon = aBoolean ifFalse: [
		showControlPolygon := aBoolean.
		self invalidate.
	].!

showGrid

	^showGrid!

showGrid: aBoolean

	showGrid = aBoolean ifFalse: [
		showGrid := aBoolean.
		self invalidate.
	].! !

!StyleView methodsFor: 'controller accessing'!

defaultControllerClass

	^StyleController! !

!StyleView methodsFor: 'displaying'!

displayOn: aGraphicsContext

	Cursor wait showWhile: [
		showGrid ifTrue: [self drawGridOn: aGraphicsContext].
		showControlPolygon ifTrue: [self drawControlPolygonOn: aGraphicsContext].
		(dragMode not or: [InputState default shiftDown]) ifTrue: [
			self model displayStyleOn: aGraphicsContext.
		].
		self drawControlPointsOn: aGraphicsContext.
		self drawStyleStartEndOn: aGraphicsContext
	].!

drawControlPointsOn: aGraphicsContext

	self currentCurve controlPoints keysAndValuesDo: [ :i :cp |
		aGraphicsContext paint:
			(i = index ifTrue: [ColorValue red] ifFalse: [ColorValue royalBlue]).
		self drawMarkerOn: aGraphicsContext at: cp label: i
	].!

drawControlPolygonOn: aGraphicsContext

	| crv controlPoints |

	crv :=self currentCurve.
	controlPoints := crv controlPoints.

	aGraphicsContext
		paint: ColorValue lightGray;
		displayPolyline: controlPoints.

	crv cyclic ifTrue: [
		aGraphicsContext
			displayLineFrom: controlPoints last
			to: controlPoints first
	].!

drawGridOn: aGraphicsContext

	| extent |

	aGraphicsContext paint: ColorValue lightGray.
	extent := self bounds extent.
	grid to: extent y by: grid do: [ :y |
		grid to: extent x by: grid do: [ :x |
			aGraphicsContext
				displayRectangle: (Rectangle origin: 0@0 corner: 1@1) at: x@y
		].
	].!

drawMarkerOn: aGraphicsContext at: coordinates label: anObject

	(Circle center: coordinates radius: radius) displayFilledOn: aGraphicsContext.
	(showControlLabels and: [anObject notNil]) ifTrue: [
		aGraphicsContext displayString: anObject asString at: 1@-1 * radius + coordinates
	].!

drawStyleStartEndOn: aGraphicsContext

	aGraphicsContext paint: ColorValue navy.
	self model styleStart notNil ifTrue: [
		self drawMarkerOn: aGraphicsContext at: self model styleStart label: 'S'.
	].
	self model styleEnd notNil ifTrue: [
		self drawMarkerOn: aGraphicsContext at: self model styleEnd label: 'E'.
	].
	
	"| coordinates |
	aGraphicsContext paint: ColorValue black.
	aGraphicsContext font: (FontDescription default pixelSize: 40).
	self model styleStart notNil ifTrue: [
		coordinates := self model styleStart.
		(Circle center: coordinates radius: radius) displayFilledOn: aGraphicsContext.
		aGraphicsContext displayString: 'S' asString at: -30@14 + coordinates
	].
	self model styleEnd notNil ifTrue: [
		coordinates := self model styleEnd.
		(Circle center: coordinates radius: radius) displayFilledOn: aGraphicsContext.
		aGraphicsContext displayString: 'E' asString at: 4@14 + coordinates
	]."! !

!StyleView methodsFor: 'save txt / print eps'!

printEpsOn: aPostScriptFile

	self model displayStyleOn: aPostScriptFile graphicsContext.
	"self drawStyleStartEndOn: aPostScriptFile graphicsContext."! !

!StyleView methodsFor: 'private'!

currentCurve

	^self model listOfCurves at: curveNr!

dragModeOff

	dragMode := false.
	self model outputParameters quality: OutputParameters qualityNormal.!

dragModeOn

	dragMode := true.
	self model outputParameters quality: OutputParameters qualityDraft.!

selectControlPointNear: aPoint

	self currentCurve controlPoints keysAndValuesDo: [:i :p |
		(p dist: aPoint) <= radius ifTrue: [index := i. ^nil]
	].
	self unselectControlPoint.
	^nil!

selectCurveNear: aPoint

	| minDist minValue minNr pOld pNew p dist |

	minDist := 10.0.
	minValue := minDist. minNr := curveNr.
	self model listOfCurves keysAndValuesDo: [ :nr :crv |
		pOld := crv controlPoints first.
		2 to: crv controlPoints size do: [ :i |
			pNew := crv controlPoints at: i.
			p := aPoint nearestPointOnLineFrom: pOld to: pNew.
			((((Rectangle vertex: pOld vertex: pNew) expandedBy: 1) containsPoint: p) and:
				[(dist := aPoint dist: p) < minValue])
					ifTrue: [minValue := dist. minNr := nr].
			pOld := pNew.
		].
	].
	self curveNr: minNr.
	^nil!

unselectControlPoint

	index := nil.! !

Point subclass: #CurvePoint
	instanceVariableNames: 'slope pressure saturation '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CurveBasics'!
CurvePoint comment:
'I am a curve point.

Instance Variables:
	pressure	Float	current pressure
	saturation	Float	current saturation
	slope		Float	current slope
'!

!CurvePoint methodsFor: 'accessing'!

coordinates

	^x@y!

coordinates: aPoint

	x := aPoint x.
	y := aPoint y.!

coordinates: aPoint1 slope: aPoint2 pressure: aNumber1 saturation: aNumber2

	self coordinates: aPoint1.
	slope := aPoint2.
	pressure := aNumber1.
	saturation := aNumber2.!

pressure

	^pressure!

pressure: aNumber

	pressure := aNumber!

saturation

	^saturation!

saturation: aNumber

	saturation := aNumber!

slope

	^slope!

slope: aPoint

	slope := aPoint! !

!CurvePoint methodsFor: 'converting'!

asPoint

	^x@y!

colorValueWithBackground: bg foreground: fg

	^bg blendWith: fg value: saturation cut!

width

	^pressure!

width: aNumber

	pressure := aNumber! !

!CurvePoint methodsFor: 'comparing'!

= anObject 

	^super = anObject and:
		[slope = anObject slope and:
			[pressure = anObject pressure and:
				[saturation = anObject saturation]]]!

hash

	^(x + y + slope + pressure + saturation) asFloat floatHash! !

!CurvePoint methodsFor: 'transforming'!

scaleBy: amount 
	"Answer the receiver scaled by the argument amount, which can be
	a Point or a scalar value."

	^self coordinates: (self coordinates scaledBy: amount)!

scaledBy: amount 
	"Answer a new object scaled by the argument amount, which can be
	a Point or a scalar value."

	^self copy scaleBy: amount!

translateBy: amount 
	"Answer the receiver translated by amount, which can be
	a Point or a scalar value."

	^self coordinates: (self coordinates translatedBy: amount)!

translatedBy: amount 
	"Answer a new object translated by amount, which can be
	a Point or a scalar value."

	^self copy translateBy: amount! !

!CurvePoint methodsFor: 'printing'!

printOn: aStream 

	aStream print: self class; space.
	self saveTxtOn: aStream!

storeOn: aStream 

	aStream
		store: self class;
		nextPutAll: ' coordinates: '; print: self coordinates;
		nextPutAll: 'slope: '; print: slope;
		nextPutAll: 'pressure: '; print: pressure;
		nextPutAll: 'saturation: '; print: saturation! !

!CurvePoint methodsFor: 'save txt / print eps'!

saveTxtOn: aStream 
	"save self in a special TXT-format"

	aStream
		nextPutAll: 'x: '; print: x; tab;
		nextPutAll: 'y: '; print: y.
	slope notNil ifTrue: [aStream tab; nextPutAll: 'slope: '; print: slope].
	pressure = 1.0 ifFalse: [aStream tab; nextPutAll: 'p: '; print: pressure].
	saturation = 1.0 ifFalse: [aStream tab; nextPutAll: 's: '; print: saturation].! !

!CurvePoint class methodsFor: 'instance creation'!

coordinates: aPoint pressure: aNumber1 saturation: aNumber2

	^self basicNew
		coordinates: aPoint
		slope: nil "if unknown"
		pressure: aNumber1
		saturation: aNumber2!

coordinates: aPoint1 slope: aPoint2 pressure: aNumber1 saturation: aNumber2

	^self basicNew
		coordinates: aPoint1
		slope: aPoint2
		pressure: aNumber1
		saturation: aNumber2! !

ExternalLanguageParser subclass: #StyleParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Style'!

!StyleParser methodsFor: 'parsing'!

boolean =

	( ( word=#true | word=#yes | word=#on )	[true] )
	| ( ( word=#false | word=#no | word=#off )	[false] )!

capStyleDef =

	( keyword=#capStyle:
	( word=#capButt			[Style capButt]
	| word=#capFlat			[Style capButt]
	| word=#capRound		[Style capRound]
	| word=#capProjecting	[Style capProjecting]
	| word=#capSquare		[Style capProjecting]
	| word=#capAngle		[Style capAngle] )
	|						[Style capButt] ) "default"!

colorDef =

	( keyword=#color: colorValueDef )
	|						[ColorValue black] "default"!

colorValueDef =

	${ number number number $}

	[makeColorValueRed:green:blue:]!

coordinatesGlobalDef =

	( keyword=#coordinatesGlobal: boolean )
	|						[false] "default"!

curve =

	pointDef
	pressureDef
	saturationDef
	variationDef
	colorDef

	[makeCurvePoints:interpolation:cyclic:pressure:saturation:variation:color:]!

interpolation1DDef =

	word=#Interpolation1D

	( (keyword=#keys: ${ number+ $})
	|								[nil] ) "no keys"

	keyword=#values: ${ number+ $}

	( ( keyword=#interpolation:
	( word=#linear1					[Interpolation linear1]
	| word=#bSpline3				[Interpolation bSpline3]
	| word=#catmull3				[Interpolation catmull3]
	| word=#chordal3				[Interpolation chordal3] ) )
	|								[Interpolation linear1] ) "default"

	( ( keyword=#cyclic: boolean )
	|								[false]) "default"

	[makeInterpolation1Dkeys:values:interpolation:cyclic:]!

orientationTangentDef =

	( keyword=#orientationTangent: boolean )
	|						[true] "default"!

penAngleDef =

	( keyword=#penAngle:
	( word=#penAngleNormal		[Style penAngleNormal]
	| word=#penAngleHorizontal		[Style penAngleHorizontal]
	| word=#penAngleVertical		[Style penAngleVertical]
	| word=#penAngleOblique		[Style penAngleOblique] )
	|								[Style penAngleNormal] ) "default"!

point =

	keyword=#x: signedNumber
	keyword=#y: signedNumber
	( keyword=#w: signedNumber
	|						[1.0] )
	( keyword=#p: number
	|						[1.0] )
	( keyword=#s: number
	|						[1.0] )

	[makePointX:y:w:p:s:]!

pointDef =

	keyword=#points: ${ point+ $}
	
	( ( keyword=#interpolation:
	( word=#bSpline			[#bSpline]
	| word=#catmull				[#catmull]
	| word=#chordal				[#chordal]
	| word=#polyline			[#polyline] ) )
	|							[#chordal] ) "default"

	( ( keyword=#cyclic: boolean )
	|							[false] ) "default"!

pressureDef =

	( keyword=#pressure:
	( interpolation1DDef
	| number)
	|						[nil] ) "default"!

pressureGlobalDef =

	( keyword=#pressureGlobal: boolean )
	|						[false] "default"!

saturationDef =

	( keyword=#saturation:
	( interpolation1DDef	
	| number)
	|						[nil] ) "default"!

saturationGlobalDef =

	( keyword=#saturationGlobal: boolean )
	|						[false] "default"!

strokeConsistencyDef =

	( keyword=#strokeConsistency: boolean )
	|						[false] "default"!

strokeGapDef =

	(keyword=#strokeGap: signedNumber)
	|						[0.0] "default"!

strokeLengthDef =

	( keyword=#strokeLength: 
	( number
	| word=#veryShort		[LogicStrokeLength veryShort]
	| word=#short			[LogicStrokeLength short]
	| word=#normal			[LogicStrokeLength normal]
	| word=#long			[LogicStrokeLength long]
	| word=#veryLong		[LogicStrokeLength veryLong]) )
	|						[LogicStrokeLength normal] "default"!

strokeNumberDef =

	( keyword=#strokeNumber:
	( number 
	| word=#nil				[nil]))
	|						[1] "default"!

strokePowerTwoDef =

	( keyword=#strokePowerTwo: boolean )
	|						[false] "default"!

style =

	word=#Style
	${
	curve+
	capStyleDef
	penAngleDef
	styleCoordinatesPriorityDef
	stylePressurePriorityDef
	styleSaturationPriorityDef
	styleColorPriorityDef
	coordinatesGlobalDef
	pressureGlobalDef
	saturationGlobalDef
	orientationTangentDef
	strokeNumberDef
	strokeLengthDef
	strokeGapDef
	strokePowerTwoDef
	strokeConsistencyDef
	styleStartDef
	styleEndDef
	thresholdPressureDef
	thresholdSaturationDef
	$}

	[makeCurves:capStyle:penAngle:coordinatesPriority:pressurePriority:saturationPriority:colorPriority:coordinatesGlobal:pressureGlobal:saturationGlobal:orientationTangent:strokeNumber:strokeLength:strokeGap:strokePowerTwo:strokeConsistency:styleStart:styleEnd:thresholdPressure:thresholdSaturation:]!

styleColorPriorityDef =

	(keyword=#styleColorPriority: number)
	|						[0.5]!

styleCoordinatesPriorityDef =

	(keyword=#styleCoordinatesPriority: number)
	|						[0.5]!

styleEndDef =

	( keyword=#styleEnd:
	( ( signedNumber $@ signedNumber )	[makePointX:y:]
	| word=#nil								[nil] ) )
	|										[nil] "default"!

stylePressurePriorityDef =

	(keyword=#stylePressurePriority: number)
	|						[0.5]!

styleSaturationPriorityDef =

	(keyword=#styleSaturationPriority: number)
	|						[0.5]!

styleStartDef =

	( keyword=#styleStart:
	( ( signedNumber $@ signedNumber )	[makePointX:y:]
	| word=#nil				[nil] ) )
	|						[nil] "default"!

thresholdPressureDef =

	(keyword=#thresholdPressure: number)
	|						[0.0] "default"!

thresholdSaturationDef =

	(keyword=#thresholdSaturation: number)
	|						[0.0] "default"!

variationDef =

	(keyword=#controlPointVariation: number)
	|						[0.0] "default"! !

!StyleParser methodsFor: 'private parsing support'!

makeColorValueRed: r green: g blue: b

	^ColorValue red: r green: g blue: b!

makeCurvePoints: cp interpolation: i cyclic: c pressure: p saturation: s variation: v color: co

	^(Curve controlPoints: cp interpolation: i cyclic: c)
		pressure: p;
		saturation: s;
		controlPointVariation: v;
		color: co!

makeCurves: listOfCurves capStyle: capStyle penAngle: penAngle coordinatesPriority: cp pressurePriority: pp saturationPriority: sp colorPriority: lp coordinatesGlobal: cg pressureGlobal: pg saturationGlobal: sg orientationTangent: ot strokeNumber: sn strokeLength: sl strokeGap: ga strokePowerTwo: s2 strokeConsistency: sc styleStart: st styleEnd: en thresholdPressure: tp thresholdSaturation: ts

	^(Style listOfCurves: listOfCurves)
		capStyle: capStyle;
		penAngle: penAngle;
		styleCoordinatesPriority: cp;
		stylePressurePriority: pp;
		styleSaturationPriority: sp;
		styleColorPriority: lp;
		coordinatesGlobal: cg;
		pressureGlobal: pg;
		saturationGlobal: sg;
		orientationTangent: ot;
		strokeNumber: sn;
		strokeLength: sl;
		strokeGap: ga;
		strokePowerTwo: s2;
		strokeConsistency: sc;
		styleStart: st;
		styleEnd: en;
		thresholdPressure: tp;
		thresholdSaturation: ts!

makeInterpolation1Dkeys: k values: v interpolation: i cyclic: c

	^k isNil
		ifTrue: [Interpolation1D values: v interpolation: i cyclic: c]
		ifFalse: [Interpolation1D keys: k values: v interpolation: i cyclic: c]!

makePointX: x y: y

	^x@y!

makePointX: x y: y w: w p: p s: s

	^ControlPoint coordinates: x@y weight: w pressure: p saturation: s! !

!StyleParser class methodsFor: 'class initialization'!

initialize
	"StyleParser initialize"

	(self organization listAtCategoryNamed: #parsing) do: [ :selector |
		self recompile: selector
	].! !

!Object methodsFor: 'save txt / print eps'!

printEps

	^self printEps: CurvePreferences defaultEpsPath, CurvePreferences defaultEpsName!

printEps: aString

	| fileName ps |

	fileName := aString.
	((fileName includes: $*) or: [fileName includes: $#]) ifTrue: [
		fileName := Dialog requestFileName: 'Name a file' default: fileName.
	].

	fileName isEmpty ifFalse: [
		ps := PostScriptFile named: fileName.
		ps doEPS: ('*.eps' match: fileName).
		"ps setLandscape: true."
		"ps copies: 2."
		Cursor write showWhile: [
			[self printEpsOn: ps] valueNowOrOnUnwindDo: [ps close]
		].
	].!

printEpsOn: aPostScriptFile

	self displayOn: aPostScriptFile graphicsContext!

saveTxt

	^self saveTxt: self class defaultTxtPath, self class defaultTxtName!

saveTxt: aString

	| fileName stream |

	fileName := aString.
	((fileName includes: $*) or: [fileName includes: $#]) ifTrue: [
		fileName := Dialog requestFileName: 'Name a file' default: fileName.
	].

	fileName isEmpty ifFalse: [
		stream := fileName asFilename writeStream.
		[self saveTxtOn: stream] valueNowOrOnUnwindDo: [stream close]
	]!

saveTxtOn: aStream
	"save self in a special TXT-format"

	self storeOn: aStream! !

!Object class methodsFor: 'preferences'!

defaultTxtName

	^CurvePreferences defaultTxtName: self!

defaultTxtPath

	^CurvePreferences defaultTxtPath: self! !

!Object class methodsFor: 'load txt'!

loadTxt

	^self loadTxt: self defaultTxtPath, self defaultTxtName!

loadTxt: aString

	| fileName stream |

	fileName := Filename replaceAllSeparatorsWithDefault: aString.
	((fileName includes: $*) or: [fileName includes: $#]) ifTrue: [
		fileName := Dialog requestFileName: 'Name a file' default: fileName.
	].

	fileName isEmpty ifFalse: [
		fileName := fileName asFilename.
		fileName exists ifTrue: [
			stream := fileName readStream.
			^[self loadTxtOn: stream] valueNowOrOnUnwindDo: [stream close].
		].
	].
	^nil!

loadTxtOn: aStream
	"load from a special TXT-format"

	self subclassResponsibility! !

Object subclass: #Interpolation
	instanceVariableNames: 'keys '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interpolation'!

!Interpolation methodsFor: 'accessing'!

keys

	^keys!

keys: aCollection
	"Attention!!
	This method manipulate the attribute 'keys' directly.
	If you're not sure what exactly you do -
	please use 'values:interpolation:' or 'withArray:interpolation:' instead"

	keys := (SortedCollection new: aCollection size)
		addAllWithoutSorting: aCollection; yourself.
	keys last < 1.0 ifFalse: [keys removeLast].! !

!Interpolation methodsFor: 'copying'!

postCopy
	"Do post-copy stuff."

	super postCopy.
	self keys: self keys copy.! !

!Interpolation methodsFor: 'printing'!

printOn: aStream

	self saveTxtOn: aStream! !

!Interpolation methodsFor: 'save txt / print eps'!

saveTxtOn: aStream
	"save self in a special TXT-format"

	aStream print: self class; cr! !

!Interpolation methodsFor: 'private'!

indexOf: u

	u < 0.0 ifTrue: [^1].
	^(self keys indexForInserting: u) - 1.!

notSupportedYet

	self error: 'not supported yet'!

tOf: u index: index

	index = self keys size ifTrue: [ "last segment"
		^u - (self keys at: index) / (1.0 - (self keys at: index))
	].
	^u - (self keys at: index) / ((self keys at: index +1) - (self keys at: index))! !

!Interpolation methodsFor: 'private keys'!

areKeysUniform

	| size |

	size := self keys size.
	self keys keysAndValuesDo: [ :i :k | i - 1.0 / size = k ifFalse: [^false] ].
	^true!

calcKeysUniform: size

	| sc |

	sc := SortedCollection new: size.
	1 to: size do: [ :i | sc add: i - 1.0 / size].
	self keys: sc.! !

!Interpolation class methodsFor: 'constants'!

bSpline2
	"quadratic"

	^3!

bSpline3
	"kubic"

	^4!

catmull3
	"kubic"

	^5!

chordal3
	"kubic"

	^6!

constant0
	"constant"

	^1!

interpolationArray

	^#(#constant0 #linear1 #bSpline2 #bSpline3 #catmull3 #chordal3)!

linear1
	"linear"

	^2! !

Object subclass: #Style
	instanceVariableNames: 'outputParameters listOfCurves capStyle penAngle styleCoordinatesPriority stylePressurePriority styleSaturationPriority styleColorPriority coordinatesGlobal pressureGlobal saturationGlobal orientationTangent strokeNumber strokeLength strokeGap strokePowerTwo strokeConsistency styleStart styleEnd thresholdPressure thresholdSaturation currentStyle currentPath currentScale currentAngle currentStyleLineX currentStyleLineY '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Style'!

!Style methodsFor: 'initialize-release'!

initialize

	super initialize.
	self outputParameters: (OutputParameters device: CurvePreferences defaultDevice).
	self capStyle: self class capButt.
	self penAngle: self class penAngleNormal.
	self styleCoordinatesPriority: 0.5.
	self stylePressurePriority: 0.5.
	self styleSaturationPriority: 0.5.
	self styleColorPriority: 0.5.
	self coordinatesGlobal: false. "true: style has effect on curve, false: style has effect on stroke"
	self pressureGlobal: false. "true: style has effect on curve, false: style has effect on stroke"
	self saturationGlobal: false. "true: style has effect on curve, false: style has effect on stroke"
	self orientationTangent: true.
	self strokeNumber: 1. "only one stroke (the whole curve)"
	self strokeLength: LogicStrokeLength normal.
	self strokeGap: 0 percent. "gapLength := strokeGap * strokeLength, default 0%"
	self strokePowerTwo: false.
	self strokeConsistency: false.
	self styleStart: nil "50@150".
	self styleEnd: nil "350@150".
	self thresholdPressure: 0.0.
	self thresholdSaturation: 0.0.!

release

	self closeAllWindows.
	super release.! !

!Style methodsFor: 'accessing'!

capStyle

	^capStyle!

capStyle: aCapStyle

	capStyle := aCapStyle!

coordinatesGlobal

	^coordinatesGlobal!

coordinatesGlobal: aBoolean

	coordinatesGlobal := aBoolean!

estimateLength

	^currentPath estimateLength!

listOfCurves

	^listOfCurves!

listOfCurves: aListOfCurves

	listOfCurves := aListOfCurves asOrderedCollection!

orientationTangent

	^orientationTangent!

orientationTangent: aBoolean

	orientationTangent := aBoolean!

outputParameters

	^outputParameters!

outputParameters: aOutputParameters

	outputParameters := aOutputParameters!

penAngle

	^penAngle!

penAngle: aBlockOrNil

	penAngle := aBlockOrNil!

pressureGlobal

	^pressureGlobal!

pressureGlobal: aBoolean

	pressureGlobal := aBoolean!

saturationGlobal

	^saturationGlobal!

saturationGlobal: aBoolean

	saturationGlobal := aBoolean!

strokeConsistency

	^strokeConsistency!

strokeConsistency: aBoolean

	strokeConsistency := aBoolean!

strokeGap

	^strokeGap!

strokeGap: aNumber

	strokeGap := aNumber!

strokeLength

	^strokeLength value!

strokeLength: aNumber

	strokeLength := aNumber!

strokeNumber

	^strokeNumber!

strokeNumber: aNumber

	strokeNumber := aNumber!

strokePowerTwo

	^strokePowerTwo!

strokePowerTwo: aBoolean

	strokePowerTwo := aBoolean!

styleColorPriority

	^styleColorPriority!

styleColorPriority: aNumber

	styleColorPriority := aNumber!

styleCoordinatesPriority

	^styleCoordinatesPriority!

styleCoordinatesPriority: aNumber

	styleCoordinatesPriority := aNumber!

styleEnd

	^styleEnd!

styleEnd: aPointOrNil

	styleEnd := aPointOrNil!

stylePressurePriority

	^stylePressurePriority!

stylePressurePriority: aNumber

	stylePressurePriority := aNumber!

styleSaturationPriority

	^styleSaturationPriority!

styleSaturationPriority: aNumber

	styleSaturationPriority := aNumber!

styleStart

	^styleStart!

styleStart: aPointOrNil

	styleStart := aPointOrNil!

thresholdPressure

	^thresholdPressure!

thresholdPressure: aNumber

	thresholdPressure := aNumber!

thresholdSaturation

	^thresholdSaturation!

thresholdSaturation: aNumber

	thresholdSaturation := aNumber! !

!Style methodsFor: 'converting'!

blendWith: aStyle value: aNumber
	"blend self with aStyle, 0 means self and 1 means aStyle"

	(self capStyle ~= aStyle capStyle or: [
	self penAngle ~= aStyle penAngle or: [
	self strokeNumber ~= aStyle strokeNumber]])
		ifTrue: [self unblendable].

	^(self class listOfCurves: (self listOfCurves blendWith: aStyle listOfCurves value: aNumber))

		outputParameters: self outputParameters ;
		capStyle: self capStyle;
		penAngle: self penAngle;
		strokeNumber: self strokeNumber;

		styleCoordinatesPriority: (self styleCoordinatesPriority blendWith: aStyle styleCoordinatesPriority value: aNumber);
		stylePressurePriority: (self stylePressurePriority blendWith: aStyle stylePressurePriority value: aNumber);
		styleSaturationPriority: (self styleSaturationPriority blendWith: aStyle styleSaturationPriority value: aNumber);
		styleColorPriority: (self styleColorPriority blendWith: aStyle styleColorPriority value: aNumber);
		coordinatesGlobal: (self coordinatesGlobal blendWith: aStyle coordinatesGlobal value: aNumber);
		pressureGlobal: (self pressureGlobal blendWith: aStyle pressureGlobal value: aNumber);
		saturationGlobal: (self saturationGlobal blendWith: aStyle saturationGlobal value: aNumber);
		orientationTangent: (self orientationTangent blendWith: aStyle orientationTangent value: aNumber);
		strokeLength: (self strokeLength blendWith: aStyle strokeLength value: aNumber);
		strokeGap: (self strokeGap blendWith: aStyle strokeGap value: aNumber);
		strokePowerTwo: (self strokePowerTwo blendWith: aStyle strokePowerTwo value: aNumber);
		strokeConsistency: (self strokeConsistency blendWith: aStyle strokeConsistency value: aNumber);
		styleStart: (self styleStart blendWith: aStyle styleStart value: aNumber);
		styleEnd: (self styleEnd blendWith: aStyle styleEnd value: aNumber);
		thresholdPressure: (self thresholdPressure blendWith: aStyle thresholdPressure value: aNumber);
		thresholdSaturation: (self thresholdSaturation blendWith: aStyle thresholdSaturation value: aNumber)! !

!Style methodsFor: 'diplaying'!

displayCurve: aPath on: aGraphicsContext

	aPath isNil ifTrue: [^self].
	aPath condenseControlPoints: self outputParameters smallestLength.
	aPath cyclic
		ifTrue: [aPath controlPoints size < 3 ifTrue: [^self]]
		ifFalse: [aPath controlPoints size < 2 ifTrue: [^self]].

	currentPath := aPath.
	self listOfCurves do: [ :s |
		currentStyle := s.
		self generateStrokes do: [ :stroke |
			self
				displayShadedPolyline: stroke
				background: CurvePreferences backgroundColor
				foreground: (currentPath color blendWith: currentStyle color
					value: self styleColorPriority)
				on: aGraphicsContext
		].
	].!

displayPath: aPath on: aGraphicsContext

	aPath isNil ifTrue: [^self].
	aPath condenseControlPoints: self outputParameters smallestLength.
	aPath cyclic
		ifTrue: [aPath controlPoints size < 3 ifTrue: [^self]]
		ifFalse: [aPath controlPoints size < 2 ifTrue: [^self]].

	self
		displayShadedPolyline:
			(aPath asPointArrayFlatness: self outputParameters maxLength)
		background: CurvePreferences backgroundColor
		foreground: aPath color
		on: aGraphicsContext!

displayStyleOn: aGraphicsContext

	self listOfCurves do: [ :s |
		self
			displayShadedPolyline: 
				(s asPointArrayFlatness: self outputParameters maxLength)
			background: CurvePreferences backgroundColor
			foreground: s color
			on: aGraphicsContext
	]! !

!Style methodsFor: 'computing'!

currentStyleLine: t

	^(currentStyleLineX pointAt: t) @ (currentStyleLineY pointAt: t)!

generateStrokes
	"the interpolation doesn't work correctly for t = 0 and t = 1
	thats why this method doesn't touch the curve ends"

	| number tl tr |
	
	tl := 0.0001 max: currentPath shortenStart / currentPath estimateLength
		* self outputParameters shorten1.
	tr := 1.0 - (0.0001 max: currentPath shortenEnd / currentPath estimateLength
		* self outputParameters shorten1).
	number := self realStrokeNumber.
	number = 1 ifTrue: [
		currentStyle controlPointVariation = 0.0 ifFalse: [currentStyle markDirty].
		^Array with: (self oneStrokeFrom: tl to: tr) "only one stroke"
	] ifFalse: [
		| deltaStroke deltaNextStroke from to newArray |

		deltaStroke := currentPath cyclic
			ifTrue: [(tr - tl) / (number * self strokeGap + number)]
			ifFalse: [(tr - tl) / ((number - 1) * self strokeGap + number)].
		deltaNextStroke := deltaStroke * self strokeGap + deltaStroke.
		from := tl.
		to := tl + deltaStroke.
		newArray := Array new: number.
		1 to: number do: [ :i |
			currentStyle controlPointVariation = 0.0 ifFalse: [currentStyle markDirty].
			newArray at: i put: (self oneStrokeFrom: from to: to).
			from := from + deltaNextStroke.
			to := to + deltaNextStroke.
		].
		^newArray
	].!

oneStrokeFrom: tl to: tr
	"the interpolation doesn't work correctly for t = 0 and t = 1
	thats why this method doesn't touch the curve ends"

	| curvePoints realStyleStart realStyleEnd xl xr sl sr flatness |

	realStyleStart := self styleStart selfOrValueIfNil: [currentStyle controlPoints first].
	realStyleEnd := self styleEnd selfOrValueIfNil: [currentStyle controlPoints last].

	currentScale := self coordinatesGlobal
		ifTrue: [currentPath estimateLength / currentStyle estimateLength
			* self styleCoordinatesPriority]
		ifFalse: [self realStrokeLength / currentStyle estimateLength
			* self styleCoordinatesPriority].
	currentAngle := (realStyleEnd - realStyleStart) theta.
	currentStyleLineX := (realStyleEnd x - realStyleStart x) @(realStyleStart x).
	currentStyleLineY := (realStyleEnd y - realStyleStart y) @(realStyleStart y).

	flatness := self outputParameters maxLength.
	sl := 0.0001. xl := self pointAt: tl stroke: sl.
	sr := 0.9999. xr := self pointAt: tr stroke: sr.

	curvePoints := OrderedCollection new: 20.
	curvePoints add: xl.
	self tLeft: tl xLeft: xl sLeft: sl tRight: tr xRight: xr sRight: sr flatness2: 2.0 * flatness toOrderedCollection: curvePoints.
	^curvePoints!

pointAt: uCurve stroke: uStroke

	| pPath sCoordinates sPressure sSaturation pLine diffVector matrix |

	pPath := currentPath pointAt: uCurve.
	sCoordinates := self coordinatesGlobal
		ifTrue: [currentStyle  coordinatesAt: uCurve]
		ifFalse: [currentStyle  coordinatesAt: uStroke].
	sPressure := self pressureGlobal
		ifTrue: [currentStyle  pressureAt: uCurve]
		ifFalse: [currentStyle  pressureAt: uStroke].
	sSaturation := self saturationGlobal
		ifTrue: [currentStyle  saturationAt: uCurve]
		ifFalse: [currentStyle  saturationAt: uStroke].
	pLine := self coordinatesGlobal
		ifTrue: [self currentStyleLine: uCurve]
		ifFalse: [self currentStyleLine: uStroke].
	diffVector := (sCoordinates - pLine) * currentScale.
	matrix := self orientationTangent
		ifTrue: [Matrix3x3 withRotation: (pPath slope theta - currentAngle) radiansToDegrees]
		ifFalse: [Matrix3x3 withRotation: (0.0 - currentAngle) radiansToDegrees].

	^CurvePoint
		coordinates: pPath coordinates + (matrix * diffVector)
		pressure: (pPath pressure blendWith: sPressure value: stylePressurePriority)
		saturation: (pPath saturation blendWith: sSaturation value: self styleSaturationPriority)! !

!Style methodsFor: 'printing'!

printOn: aStream

	self saveTxtOn: aStream! !

!Style methodsFor: 'save txt / print eps'!

saveTxtOn: aStream
	"save self in a special TXT-format"

	aStream print: self class; nextPutAll: ' {'; cr.

	self listOfCurves do: [ :crv | crv saveTxtOn: aStream].

	self capStyle = self class capButt ifFalse: [
		aStream tab; nextPutAll: 'capStyle: ';
			nextPutAll: (self class capArray at: self capStyle) asString; cr.
	].

	self styleCoordinatesPriority = 0.5 ifFalse: [
		aStream tab; nextPutAll: 'styleCoordinatesPriority: '.
		self styleCoordinatesPriority saveTxtOn: aStream.
		aStream cr.
	].

	self stylePressurePriority = 0.5 ifFalse: [
		aStream tab; nextPutAll: 'stylePressurePriority: '.
		self stylePressurePriority saveTxtOn: aStream.
		aStream cr.
	].

	self styleSaturationPriority = 0.5 ifFalse: [
		aStream tab; nextPutAll: 'styleSaturationPriority: '.
		self styleSaturationPriority saveTxtOn: aStream.
		aStream cr.
	].

	self styleColorPriority = 0.5 ifFalse: [
		aStream tab; nextPutAll: 'styleColorPriority: '.
		self styleColorPriority saveTxtOn: aStream.
		aStream cr.
	].

	self coordinatesGlobal ifTrue: [aStream tab; nextPutAll: 'coordinatesGlobal: true'; cr].

	self pressureGlobal ifTrue: [aStream tab; nextPutAll: 'pressureGlobal: true'; cr].

	self saturationGlobal ifTrue: [aStream tab; nextPutAll: 'saturationGlobal: true'; cr].

	self orientationTangent ifFalse: [aStream tab; nextPutAll: 'orientationTangent: false'; cr].

	self strokeNumber = 1 ifFalse: [
		aStream tab; nextPutAll: 'strokeNumber: '; store: self strokeNumber; cr.
	].

	self strokeLength = LogicStrokeLength normal value ifFalse: [
		aStream tab; nextPutAll: 'strokeLength: '.
		strokeLength saveTxtOn: aStream.
		aStream cr.
	].

	self strokeGap = 0.0 ifFalse: [
		aStream tab; nextPutAll: 'strokeGap: '; store: self strokeGap; cr.
	].

	self strokePowerTwo ifTrue: [aStream tab; nextPutAll: 'strokePowerTwo: true'; cr].

	self strokeConsistency ifTrue: [aStream tab; nextPutAll: 'strokeConsistency: true'; cr].

	self styleStart isNil ifFalse: [
		aStream tab; nextPutAll: 'styleStart: ';
			store: self styleStart x; nextPut: $@; store: self styleStart y; cr.
	].

	self styleEnd isNil ifFalse: [
		aStream tab; nextPutAll: 'styleEnd: ';
			store: self styleEnd x; nextPut: $@; store: self styleEnd y; cr.
	].

	self thresholdPressure = 0.0 ifFalse: [
		aStream tab; nextPutAll: 'thresholdPressure: '.
		self thresholdPressure saveTxtOn: aStream.
		aStream cr.
	].

	self thresholdSaturation = 0.0 ifFalse: [
		aStream tab; nextPutAll: 'thresholdSaturation: '.
		self thresholdSaturation saveTxtOn: aStream.
		aStream cr.
	].

	aStream nextPut: $}; cr! !

!Style methodsFor: 'enumerating'!

verticesDo: aBlock
	"Evaluate the supplied block for every vertex in the receiver."

	self listOfCurves do: [ :crv | crv verticesDo: aBlock]! !

!Style methodsFor: 'edit'!

closeAllWindows

	self dependents do: [ :d |
		(d isKindOf: View) ifTrue: [d topComponent controller closeAndUnschedule].
	].!

edit

	ScheduledWindow new
		"damageRepairPolicy: DoubleBufferingWindowDisplayPolicy new;"
		label: 'Style';
		minimumSize: 400@300;
		component: (StyleView model: self);
		background: ColorValue white;
		open.!

editStyleProperties

	| dialog builder doEdit capStyleMenuEditing1 penAngleMenuEditing1 styleCoordinatesPriority1 stylePressurePriority1 styleSaturationPriority1 styleColorPriority1 coordinatesGlobal1 pressureGlobal1 saturationGlobal1 orientationTangent1 strokeNumber1 strokeLength1 strokeGap1 strokePowerTwo1 strokeConsistency1 thresholdPressure1 thresholdSaturation1 |

	dialog := SimpleDialog new.
	builder := dialog builder.

	builder aspectAt: #capStyleMenu put: self class capStyleMenu.
	builder aspectAt: #penAngleMenu put: self class penAngleMenu.
	builder aspectAt: #capStyleMenuEditing put: (capStyleMenuEditing1 := (self class capArray at: self capStyle) asValue).
	builder aspectAt: #penAngleMenuEditing put: (penAngleMenuEditing1 := #penAngleNormal asValue).
	builder aspectAt: #styleCoordinatesPriority put: (styleCoordinatesPriority1 := self styleCoordinatesPriority asValue).
	builder aspectAt: #stylePressurePriority put: (stylePressurePriority1 := self stylePressurePriority asValue).
	builder aspectAt: #styleSaturationPriority put: (styleSaturationPriority1 := self styleSaturationPriority asValue).
	builder aspectAt: #styleColorPriority put: (styleColorPriority1 := self styleColorPriority asValue).
	builder aspectAt: #coordinatesGlobal put: (coordinatesGlobal1 := self coordinatesGlobal asValue).
	builder aspectAt: #pressureGlobal put: (pressureGlobal1 := self pressureGlobal asValue).
	builder aspectAt: #saturationGlobal put: (saturationGlobal1 := self saturationGlobal asValue).
	builder aspectAt: #strokeNumber put: (strokeNumber1 := self strokeNumber asValue).
	builder aspectAt: #strokeLength put: (strokeLength1 := self strokeLength value asValue).
	builder aspectAt: #strokeGap put: (strokeGap1 := self strokeGap asValue).
	builder aspectAt: #strokePowerTwo put: (strokePowerTwo1 := self strokePowerTwo asValue).
	builder aspectAt: #strokeConsistency put: (strokeConsistency1 := self strokeConsistency asValue).
	builder aspectAt: #thresholdPressure put: (thresholdPressure1 := self thresholdPressure asValue).
	builder aspectAt: #thresholdSaturation put: (thresholdSaturation1 := self thresholdSaturation asValue).
	builder aspectAt: #orientationTangent put: (orientationTangent1 := self orientationTangent asValue).

	builder aspectAt: #doAccept put: [doEdit := true. dialog close].
	builder aspectAt: #doCancel put: [doEdit := false. dialog close].

	dialog openFrom: (UISpecification from: self class stylePropertiesDialogSpec).

	doEdit ifTrue: [
		self capStyle: (self class perform: capStyleMenuEditing1 value).
		self penAngle: (self class perform: penAngleMenuEditing1 value).
		self styleCoordinatesPriority: styleCoordinatesPriority1 value.
		self stylePressurePriority: stylePressurePriority1 value.
		self styleSaturationPriority: styleSaturationPriority1 value.
		self styleColorPriority: styleColorPriority1 value.
		self coordinatesGlobal: coordinatesGlobal1 value.
		self pressureGlobal: pressureGlobal1 value.
		self saturationGlobal: saturationGlobal1 value.
		self strokeNumber: strokeNumber1 value.
		self strokeLength: strokeLength1 value.
		self strokeGap: strokeGap1 value.
		self strokePowerTwo: strokePowerTwo1 value.
		self strokeConsistency: strokeConsistency1 value.
		self thresholdPressure: thresholdPressure1 value.
		self thresholdSaturation: thresholdSaturation1 value.
		self orientationTangent: orientationTangent1 value.
	]! !

!Style methodsFor: 'private'!

getContour: vertices

	| array firstIndex lastIndex cp0 cp1 cp2 normal normal1 normal2 width1 cyclic |

	array := Array new: 2 * vertices size.
	firstIndex := 1.
	lastIndex := array size.
	width1 := self outputParameters width1.

	self penAngle notNil ifTrue: [
		1 to: vertices size do: [ :i |
			cp1 := vertices at: i.
			normal := cp1 width * width1 / 2.0 * self penAngle value.
			array at: firstIndex put: cp1 coordinates + normal. firstIndex := firstIndex + 1.
			array at: lastIndex put: cp1 coordinates - normal. lastIndex := lastIndex - 1.
		].
		^array.
	].

	cp0 := vertices at: 1.
	cp1 := vertices at: 2.

	cyclic := (cp0 coordinates dist: vertices last coordinates) <
		self outputParameters smallestLength.

	normal1 := (cp1 coordinates - cp0 coordinates) normal.
	normal := cyclic 
		ifTrue: [cp2 := vertices at: vertices size - 1.
			normal2 := (cp0 coordinates - cp2 coordinates) normal.
			normal1 + normal2 / 2.0]
		ifFalse: [normal1].

	normal := normal * cp0 width * width1 / 2.0.
	array at: firstIndex put: cp0 coordinates + normal. firstIndex := firstIndex + 1.
	array at: lastIndex put: cp0 coordinates - normal. lastIndex := lastIndex - 1.

	2 to: vertices size do: [ :i |
		i = vertices size ifTrue: [
			normal := cyclic 
				ifTrue: [cp2 := vertices at: 2.
					normal2 := (cp2 coordinates - cp1 coordinates) normal.
					normal1 + normal2 / 2.0]
				ifFalse: [normal1].
		] ifFalse: [
			cp2 := vertices at: (i + 1).
			normal2 := (cp2 coordinates - cp1 coordinates) normal.
			normal := normal1 + normal2 / 2.0.
		].

		normal := normal * cp1 width * width1 / 2.0.
		array at: firstIndex put: cp1 coordinates + normal. firstIndex := firstIndex + 1.
		array at: lastIndex put: cp1 coordinates - normal. lastIndex := lastIndex - 1.

		cp0 := cp1. cp1 := cp2.
		normal1 := normal2.
	].
	^array!

realStrokeLength

	^currentPath cyclic
		ifTrue: [currentPath estimateLength /
			(1.0 + self strokeGap * self realStrokeNumber)]
		ifFalse: [currentPath estimateLength /
			(1.0 + self strokeGap * self realStrokeNumber - self strokeGap)]!

realStrokeNumber

	| x |

	x := (self strokeNumber selfOrValueIfNil:
		(currentPath cyclic
			ifTrue: [currentPath estimateLength /
				(self outputParameters stroke1 * self strokeLength) /
				(1.0 + self strokeGap)]
			ifFalse: [currentPath estimateLength /
				(self outputParameters stroke1 * self strokeLength) + self strokeGap /
				(1.0 + self strokeGap)]) rounded) max: 1.
	self strokePowerTwo ifTrue: [^x roundToAPowerOfTwo].
	^x!

tLeft: tl xLeft: xl sLeft: sl tRight: tr xRight: xr sRight: sr flatness2: flatness2 toOrderedCollection: oc

	| tm xm sm |

	tm := tl + tr / 2.0.
	sm := sl + sr / 2.0.
	xm := self pointAt: tm stroke: sm.

	((xl dist: xm) > flatness2 or: [(xm dist: xr) > flatness2])
			ifTrue: [
				self tLeft: tl xLeft: xl sLeft: sl tRight: tm xRight: xm sRight: sm flatness2: flatness2 toOrderedCollection: oc.
				self tLeft: tm xLeft: xm sLeft: sm tRight: tr xRight: xr sRight: sr flatness2: flatness2 toOrderedCollection: oc
			]
			ifFalse: [
				oc add: xm.
				oc add: xr
			].! !

!Style methodsFor: 'private display'!

displayCapStyle: aCurvePoint direction: aPoint color: aColorValue on: aGraphicsContext

	| d |

	d := aCurvePoint width * self outputParameters width1. "d = 2 * r"
	d > self outputParameters lineOptimization ifTrue: [
		| r direction p0 p1 p2 p3 |

		r := d / 2.0.
		direction := aPoint unitVector.

		aGraphicsContext paint: aColorValue.

		self capStyle = self class capRound ifTrue: [
			self outputParameters quality = OutputParameters qualityDraft ifTrue: [
				aGraphicsContext
					displayArcBBoxOrigin: aCurvePoint coordinates - r
					extent: d @ d
					startAngle: 0.0
					sweepAngle: 360.0
			] ifFalse: [
				aGraphicsContext
					displayWedgeBBoxOrigin: aCurvePoint coordinates - r
					extent: d @ d
					startAngle: 0.0
					sweepAngle: 360.0
			]
		].

		self capStyle = self class capProjecting ifTrue: [
			p0 := aCurvePoint coordinates + (r * direction normal).
			p3 := aCurvePoint coordinates - (r * direction normal).
			p1 := p0 + (r * direction).
			p2 := p3 + (r * direction).
			self outputParameters quality = OutputParameters qualityDraft ifTrue: [
				aGraphicsContext displayPolyline: (Array with: p0 with: p1 with: p2 with: p3)
			] ifFalse: [
				aGraphicsContext displayPolygon: (Array with: p0 with: p1 with: p2 with: p3)
			].
		].

		self capStyle = self class capAngle ifTrue: [
			p0 := aCurvePoint coordinates + (r * direction normal).
			p1 := aCurvePoint coordinates + (r * direction).
			p2 := aCurvePoint coordinates - (r * direction normal).
			self outputParameters quality = OutputParameters qualityDraft ifTrue: [
				aGraphicsContext displayPolyline: (Array with: p0 with: p1 with: p2)
			] ifFalse: [
				aGraphicsContext displayPolygon: (Array with: p0 with: p1 with: p2)
			].
		].
	].!

displayShadedPolyline: vertices background: bg foreground: fg on: aGraphicsContext

	| "firstPressure samePressure" firstSaturation sameSaturation |

	vertices
		if: [ :cp |
			"cp pressure = firstPressure ifFalse: [samePressure := false]."
			cp saturation = firstSaturation ifFalse: [sameSaturation := false].
			cp pressure >= self thresholdPressure and:
				[cp saturation >= self thresholdSaturation]
		]
		doWithSequence: [ :s |
			s size > 1 ifTrue: [
				sameSaturation
					ifTrue: [self
						primDisplayNonShadedPolyline: s
						background: bg
						foreground: fg
						on: aGraphicsContext]
					ifFalse: [self
						primDisplayShadedPolyline: s
						background: bg
						foreground: fg
						on: aGraphicsContext].
			].
		]
		doWithFirst: [ :cp |
			"firstPressure := cp pressure. samePressure := true."
			firstSaturation := cp saturation. sameSaturation := true.
		].!

primDisplayNonShadedPolyline: vertices background: bg foreground: fg on: aGraphicsContext

	| cp0 cp1 cp2 cp3 c |

	aGraphicsContext lineWidth: (self outputParameters lineOptimization).

	"LineEnd"
	cp0 := vertices at: 1.
	cp1 := vertices at: 2.
	cp2 := vertices at: vertices size - 1.
	cp3 := vertices at: vertices size.
	c := cp0 colorValueWithBackground: bg foreground: fg. "it doesn't matter which point"

	(cp0 coordinates dist: cp3 coordinates) < self outputParameters smallestLength
		ifFalse: [ "not cyclic"
			self displayCapStyle: cp0 direction: cp0 - cp1 color: c on: aGraphicsContext.
			self displayCapStyle: cp3 direction: cp3 - cp2 color: c on: aGraphicsContext.
		].

	aGraphicsContext paint: c.
	self outputParameters quality = OutputParameters qualityDraft
		ifTrue: [aGraphicsContext displayPolyline: (self getContour: vertices)]
		ifFalse: [aGraphicsContext displayPolygon: (self getContour: vertices)].!

primDisplayShadedPolyline: vertices background: bg foreground: fg on: aGraphicsContext

	| array cp0 cp1 cp2 cp3 wcp0 wcp1 p1 p2 p3 p4 width1 firstIndex lastIndex |

	width1 := self outputParameters width1.
	aGraphicsContext lineWidth: (self outputParameters lineOptimization).

	"LineEnd"
	cp0 := vertices at: 1. wcp0 := cp0 width * width1.
	cp1 := vertices at: 2. wcp1 := cp1 width * width1.
	cp2 := vertices at: vertices size - 1.
	cp3 := vertices at: vertices size.

	(cp0 coordinates dist: cp3 coordinates) < self outputParameters smallestLength
		ifFalse: [ "not cyclic"
			self
				displayCapStyle: cp0
				direction: cp0 - cp1
				color: (cp0 colorValueWithBackground: bg foreground: fg)
				on: aGraphicsContext.
			self
				displayCapStyle: cp3
				direction: cp3 - cp2
				color: (cp3 colorValueWithBackground: bg foreground: fg)
				on: aGraphicsContext.
		].

	array := self getContour: vertices.
	firstIndex := 1.
	lastIndex := array size.

	cp0 := vertices at: 1. wcp0 := cp0 width * width1.
	p1 := array at: firstIndex.
	p2 := array at: lastIndex.
	
	2 to: vertices size do: [ :i |
		cp1 := vertices at: i. wcp1 := cp1 width * width1.
		firstIndex := firstIndex + 1. p3 := array at: firstIndex.
		lastIndex := lastIndex - 1. p4 := array at: lastIndex.

		aGraphicsContext paint: (
			(cp0 colorValueWithBackground: bg foreground: fg) blendWith: 
			(cp1 colorValueWithBackground: bg foreground: fg)).

		(wcp0 > self outputParameters lineOptimization or: [wcp1 > self outputParameters lineOptimization]) ifTrue: [
			self outputParameters quality = OutputParameters qualityDraft  ifTrue: [
				aGraphicsContext displayPolyline: (Array with: p1 with: p3 with: p4 with: p2)
			] ifFalse: [
				aGraphicsContext displayPolygon: (Array with: p1 with: p3 with: p4 with: p2)
			].
		] ifFalse: [
			aGraphicsContext displayLineFrom: cp0 to: cp1
		].
		cp0 := cp1. wcp0 := wcp1.
		p1 := p3. p2 := p4.
	].! !

!Style class methodsFor: 'instance creation'!

listOfCurves: aListOfCurves

	^self new listOfCurves: aListOfCurves!

new

	^super new initialize! !

!Style class methodsFor: 'interface specs'!

stylePropertiesDialogSpec
	"UIPainter new openOnClass: self andSelector: #stylePropertiesDialogSpec"

	<resource: #canvas>
	^#(#FullSpec 
		#window: 
		#(#WindowSpec 
			#label: 'style properties' 
			#min: #(#Point 447 432 ) 
			#max: #(#Point 447 432 ) 
			#bounds: #(#Rectangle 105 34 552 466 ) ) 
		#component: 
		#(#SpecCollection 
			#collection: #(
				#(#GroupBoxSpec 
					#layout: #(#Rectangle 16 8 216 96 ) 
					#label: 'i don''t know' ) 
				#(#GroupBoxSpec 
					#layout: #(#Rectangle 8 104 216 256 ) 
					#label: 'priority' ) 
				#(#GroupBoxSpec 
					#layout: #(#Rectangle 232 288 432 352 ) 
					#label: 'orientation' ) 
				#(#GroupBoxSpec 
					#layout: #(#Rectangle 232 8 432 184 ) 
					#label: 'stroke' ) 
				#(#GroupBoxSpec 
					#layout: #(#Rectangle 232 192 432 280 ) 
					#label: 'threshold' ) 
				#(#GroupBoxSpec 
					#layout: #(#Rectangle 16 264 216 376 ) 
					#label: 'style global influence' ) 
				#(#LabelSpec 
					#layout: #(#Point 32 32 ) 
					#label: 'cap style' ) 
				#(#LabelSpec 
					#layout: #(#Point 32 64 ) 
					#label: 'pen angle' ) 
				#(#LabelSpec 
					#layout: #(#Point 32 128 ) 
					#label: 'coordinates' ) 
				#(#LabelSpec 
					#layout: #(#Point 32 160 ) 
					#label: 'pressure' ) 
				#(#LabelSpec 
					#layout: #(#Point 32 192 ) 
					#label: 'saturation' ) 
				#(#LabelSpec 
					#layout: #(#Point 248 32 ) 
					#label: 'number' ) 
				#(#LabelSpec 
					#layout: #(#Point 248 64 ) 
					#label: 'length' ) 
				#(#LabelSpec 
					#layout: #(#Point 248 96 ) 
					#label: 'gap' ) 
				#(#LabelSpec 
					#layout: #(#Point 248 216 ) 
					#label: 'pressure' ) 
				#(#LabelSpec 
					#layout: #(#Point 248 248 ) 
					#label: 'saturation' ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 136 392 200 416 ) 
					#model: #doAccept 
					#label: 'Apply' 
					#isDefault: true 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 248 392 312 416 ) 
					#model: #doCancel 
					#label: 'Cancel' 
					#defaultable: true ) 
				#(#MenuButtonSpec 
					#layout: #(#Rectangle 112 32 208 56 ) 
					#model: #capStyleMenuEditing 
					#menu: #capStyleMenu ) 
				#(#MenuButtonSpec 
					#layout: #(#Rectangle 112 64 208 88 ) 
					#model: #penAngleMenuEditing 
					#menu: #penAngleMenu ) 
				#(#InputFieldSpec 
					#layout: #(#Rectangle 136 128 208 152 ) 
					#model: #styleCoordinatesPriority 
					#alignment: #right 
					#type: #number ) 
				#(#InputFieldSpec 
					#layout: #(#Rectangle 136 160 208 184 ) 
					#model: #stylePressurePriority 
					#alignment: #right 
					#type: #number ) 
				#(#InputFieldSpec 
					#layout: #(#Rectangle 136 192 208 216 ) 
					#model: #styleSaturationPriority 
					#alignment: #right 
					#type: #number ) 
				#(#RadioButtonSpec 
					#layout: #(#Point 248 320 ) 
					#model: #orientationTangent 
					#label: 'tangent' 
					#select: true ) 
				#(#RadioButtonSpec 
					#layout: #(#Point 344 320 ) 
					#model: #orientationTangent 
					#label: 'fixed' 
					#select: false ) 
				#(#InputFieldSpec 
					#layout: #(#Rectangle 352 32 424 56 ) 
					#model: #strokeNumber 
					#alignment: #right 
					#type: #number ) 
				#(#InputFieldSpec 
					#layout: #(#Rectangle 352 64 424 88 ) 
					#model: #strokeLength 
					#alignment: #right 
					#type: #number ) 
				#(#InputFieldSpec 
					#layout: #(#Rectangle 352 96 424 120 ) 
					#model: #strokeGap 
					#alignment: #right 
					#type: #number ) 
				#(#InputFieldSpec 
					#layout: #(#Rectangle 352 216 424 240 ) 
					#model: #thresholdPressure 
					#alignment: #right 
					#type: #number ) 
				#(#InputFieldSpec 
					#layout: #(#Rectangle 352 248 424 272 ) 
					#model: #thresholdSaturation 
					#alignment: #right 
					#type: #number ) 
				#(#CheckBoxSpec 
					#layout: #(#Point 32 296 ) 
					#model: #coordinatesGlobal 
					#label: 'coordinates' ) 
				#(#CheckBoxSpec 
					#layout: #(#Point 32 320 ) 
					#model: #pressureGlobal 
					#label: 'pressure' ) 
				#(#CheckBoxSpec 
					#layout: #(#Point 32 344 ) 
					#model: #saturationGlobal 
					#label: 'saturation' ) 
				#(#CheckBoxSpec 
					#layout: #(#Point 248 128 ) 
					#model: #strokePowerTwo 
					#label: 'a power of 2' ) 
				#(#CheckBoxSpec 
					#layout: #(#Point 248 152 ) 
					#model: #strokeConsistency 
					#label: 'consistency' ) 
				#(#LabelSpec 
					#layout: #(#Point 32 224 ) 
					#label: 'color' ) 
				#(#InputFieldSpec 
					#layout: #(#Rectangle 136 224 208 248 ) 
					#model: #styleColorPriority 
					#alignment: #right 
					#type: #number ) ) ) )! !

!Style class methodsFor: 'resources'!

capStyleMenu
	"UIMenuEditor new openOnClass: self andSelector: #capStyleMenu"

	<resource: #menu>
	^#(#Menu #(
			#(#MenuItem 
				#rawLabel: 'butt' 
				#value: #capButt ) 
			#(#MenuItem 
				#rawLabel: 'round' 
				#value: #capRound ) 
			#(#MenuItem 
				#rawLabel: 'projecting' 
				#value: #capProjecting ) 
			#(#MenuItem 
				#rawLabel: 'angle' 
				#value: #capAngle ) ) #(4 ) nil ) decodeAsLiteralArray!

penAngleMenu
	"UIMenuEditor new openOnClass: self andSelector: #penAngleMenu"

	<resource: #menu>
	^#(#Menu #(
			#(#MenuItem 
				#rawLabel: 'normal' 
				#value: #penAngleNormal ) 
			#(#MenuItem 
				#rawLabel: 'horizontal' 
				#value: #penAngleHorizontal ) 
			#(#MenuItem 
				#rawLabel: 'vertical' 
				#value: #penAngleVertical ) 
			#(#MenuItem 
				#rawLabel: 'oblique' 
				#value: #penAngleOblique ) ) #(4 ) nil ) decodeAsLiteralArray! !

!Style class methodsFor: 'constants'!

capAngle
	"Answer the integer constant representing the angle cap style."
	^4!

capArray

	^#(#capButt #capRound #capProjecting #capAngle)!

capButt
	"Answer the integer constant representing the butt cap style."
	^1!

capProjecting
	"Answer the integer constant representing the projecting cap style."
	^3!

capRound
	"Answer the integer constant representing the round cap style."
	^2!

penAngleHorizontal

	^[1.0@0.0]!

penAngleNormal

	^nil!

penAngleOblique

	^[0.866025@-0.5]	"Point r: 1.0 theta: -30.0 degreesToRadians"!

penAngleVertical

	^[0.0@1.0]! !

!Style class methodsFor: 'load txt'!

loadTxtOn: aStream
	"load from a special TXT-format"

	^(StyleParser parse: aStream nextChunk as: #style) at: 1! !

!Style class methodsFor: 'examples'!

example
	"Style example"

	Style loadTxt edit!

test
	"Style test"

	| gc st pa |

	gc := ScheduledControllers activeController view graphicsContext.
	st := Style loadTxt: Style defaultTxtPath , 'mad1.crv'.
	st outputParameters: (OutputParameters device: CurvePreferences defaultDevice).
	pa := Path controlPoints: (OrderedCollection
					with: (ControlPoint coordinates: 50 @ 50)
					with: (ControlPoint coordinates: 100 @ 100)
					with: (ControlPoint coordinates: 200 @ 100)
					with: (ControlPoint coordinates: 250 @ 50))
				interpolation: #chordal.
	st displayCurve: pa on: gc! !

Object subclass: #Curve
	instanceVariableNames: 'controlPoints interpolation cyclic pressure saturation controlPointVariation color dirty estimateLength collectionIndex arrayLength arrayX arrayY arrayWeight arrayPressure arraySaturation '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CurveBasics'!

!Curve methodsFor: 'initialize-release'!

initialize

	super initialize.
	self pressure: nil.
	self saturation: nil.
	self controlPointVariation: 0.0.
	self color: ColorValue black.
	self markDirty.! !

!Curve methodsFor: 'accessing'!

color

	^color!

color: aColorValue

	color := aColorValue.
	self markDirty.!

controlPoints

	^controlPoints!

controlPoints: listOfPoints

	controlPoints := listOfPoints.
	self markDirty.!

controlPoints: listOfPoints interpolation: interpolationProperty cyclic: isCyclic

	controlPoints := listOfPoints.
	interpolation := interpolationProperty.
	cyclic := isCyclic.
	self markDirty.!

controlPointVariation

	^controlPointVariation!

controlPointVariation: aNumber

	controlPointVariation := aNumber.
	self markDirty.!

cyclic

	^cyclic!

cyclic: isCyclic

	cyclic := isCyclic.
	self markDirty.!

estimateLength

	dirty ifTrue: [self prepareInterpolation].
	^estimateLength!

interpolation

	^interpolation!

interpolation: interpolationProperty

	interpolation := interpolationProperty.
	self markDirty.!

markDirty

	dirty := true.!

pressure

	^pressure!

pressure: aBlockOrInterpolation

	pressure := aBlockOrInterpolation.
	self markDirty.!

saturation

	^saturation!

saturation: aBlockOrInterpolation

	saturation := aBlockOrInterpolation.
	self markDirty.! !

!Curve methodsFor: 'computing'!

coordinatesAt: u

	| i t weight |

	dirty ifTrue: [self prepareInterpolation].

	i := self indexOf: u.
	t :=	self tOf: u index: i.

	weight := (arrayWeight at: i) pointAt: t.

	^((arrayX at: i) pointAt: t) @ ((arrayY at: i) pointAt: t) / weight!

estimateLengthSegment: index
	"This is only a rough estimate of the curve length. IMO it is immpossible
	to calculate the exact length with a normal effort. I hope this method will
	be updated soon"

	| lengthPart resolution x y w pOld pNew |

	resolution := 0.2. "5 lines = 6 points"
	x := arrayX at: index.
	y := arrayY at: index.
	w := arrayWeight at: index.
	lengthPart := 0.0.
	"pOld := (self computePoint: 0.0 segment: index) coordinates."
	pOld := ((x pointAt: 0.0) @ (y pointAt: 0.0)) / (w pointAt: 0.0).
	resolution to: 1.0 + (resolution / 2.0) by: resolution do: [:t |
		"pNew := (self computePoint: t segment: index) coordinates."
		pNew := ((x pointAt: t) @ (y pointAt: t)) / (w pointAt: t).
		lengthPart := lengthPart + (pOld dist: pNew).
		pOld := pNew.
	].
	^lengthPart!

pointAt: u

	| i t weight p s |

	dirty ifTrue: [self prepareInterpolation].

	i := self indexOf: u.
	t :=	self tOf: u index: i.

	weight := (arrayWeight at: i) pointAt: t.
	p := pressure isNil
		ifTrue: [(arrayPressure at: i) pointAt: t]
		ifFalse: [pressure pointAt: u].
	s := saturation isNil
		ifTrue: [(arraySaturation at: i) pointAt: t]
		ifFalse: [saturation pointAt: u].

	^CurvePoint
		coordinates: ((arrayX at: i) pointAt: t) @ ((arrayY at: i) pointAt: t) / weight
		slope: ((arrayX at: i) derivativeAt: t) @ ((arrayY at: i) derivativeAt: t) / weight
		pressure: p
		saturation: s!

prepareInterpolation

	| values size sizeNew p0 p1 p2 p3 lengthPart matrixC matrixAttribut c0 c1 c2 c3 d0 d1 d2 vx vy |

	dirty := false.
	size := controlPoints size.

	"endpoint"
	cyclic ifTrue: [ "cylic"
		sizeNew := size + 3.
		values := Array new: sizeNew.
		values replaceFrom: 1 to: size with: controlPoints startingAt: 1.
		values replaceFrom: size + 1 to: size + 3 with: controlPoints startingAt: 1.
	]
	ifFalse: [ "double or triple"
		| extras |
		extras := interpolation = #bSpline ifTrue: [2] ifFalse: [1].
		sizeNew := 2 * extras + size.
		values := Array new: sizeNew.
		1 to: extras do: [:i |
			values at: i put: (controlPoints at: 1).
			values at: size+extras+i put: (controlPoints at: size)
		].
		values replaceFrom: extras+1 to: extras+size with: controlPoints.
	].

	"interpolationarrays"
	size := sizeNew - 3.
	collectionIndex := SortedCollection new: size.
	arrayLength := Array new: size.
	arrayX := Array new: size.
	arrayY := Array new: size.
	arrayWeight := Array new: size.
	arrayPressure := pressure isNil ifTrue: [Array new: size] ifFalse: [nil].
	arraySaturation := saturation isNil ifTrue: [Array new: size] ifFalse: [nil].
	lengthPart := 0.0.
	p1 := (values at: 1) variate: controlPointVariation.
	p2 := (values at: 2) variate: controlPointVariation.
	p3 := (values at: 3) variate: controlPointVariation.

	matrixC := interpolation = #chordal
		ifTrue: [Matrix4x4 bezier]
		ifFalse: [Matrix4x4 perform: interpolation].
	matrixAttribut := Matrix4x4 catmull.
	
	1 to: size do: [ :i |
		p0 := p1. p1 := p2. p2 := p3. p3 := (values at: i + 3) variate: controlPointVariation.
		c0 := p0 * p0 weight. c1 := p1 * p1 weight. c2 := p2 * p2 weight. c3 := p3 * p3 weight.

		interpolation = #chordal
			ifTrue: [ "let's rock"
				d0 := c0 dist: c1. d1 := c1 dist: c2. d2 := c2 dist: c3.

				vx := (c1 x @ (c1 x + ((c2 x - c0 x) / 3.0 * d1 / (d0 + d1))) @
					(c2 x  + ((c1 x - c3 x) / 3.0 * d1 / (d1 + d2))) @ c2 x).
				vy := (c1 y @ (c1 y + ((c2 y - c0 y) / 3.0 * d1 / (d0 + d1))) @
					(c2 y  + ((c1 y - c3 y) / 3.0 * d1 / (d1 + d2))) @ c2 y).
			]
			ifFalse: [
				vx := c0 x @ c1 x @ c2 x @ c3 x.
				vy := c0 y @ c1 y @ c2 y @ c3 y.
			].
		arrayX at: i put: (matrixC * vx) polynomialSimplify.
		arrayY at: i put: (matrixC * vy) polynomialSimplify.
		arrayWeight at: i put: (matrixC * (p0 weight @ p1 weight @ p2 weight @ p3
			weight)) polynomialSimplify.
		pressure isNil ifTrue: [arrayPressure at: i put:
			(matrixAttribut * (p0 pressure value @ p1 pressure value @
			p2 pressure value @ p3 pressure value)) polynomialSimplify.
		].
		saturation isNil ifTrue: [arraySaturation at: i put:
			(matrixAttribut * (p0 saturation value @ p1 saturation value @
			p2 saturation value @ p3 saturation value)) polynomialSimplify.
		].
		arrayLength at: i put: (self estimateLengthSegment: i).
		lengthPart := lengthPart + (arrayLength at: i).
	].
	estimateLength := lengthPart.

	"arrayLength -> collectionIndex"
	lengthPart := 0.0.
	1 to: size do: [ :i |
		collectionIndex add: lengthPart / estimateLength.
		lengthPart := lengthPart + (arrayLength at: i).
	].!

pressureAt: u

	| i t |

	dirty ifTrue: [self prepareInterpolation].

	^pressure isNil
		ifTrue: [i := self indexOf: u. t := self tOf: u index: i. (arrayPressure at: i) pointAt: t]
		ifFalse: [pressure pointAt: u].!

saturationAt: u

	| i t |

	dirty ifTrue: [self prepareInterpolation].

	^saturation isNil
		ifTrue: [i := self indexOf: u. t := self tOf: u index: i. (arraySaturation at: i) pointAt: t]
		ifFalse: [saturation pointAt: u].! !

!Curve methodsFor: 'converting'!

asPointArrayFlatness: flatness

	| newCollection tl xl tr xr |

	tl := 0.0001. xl := self pointAt: tl.
	tr := 0.9999. xr := self pointAt: tr.

	newCollection := OrderedCollection new.
	newCollection add: xl.
	self tLeft: tl xLeft: xl tRight: tr xRight: xr flatness2: 2.0 * flatness toOrderedCollection: newCollection.

	^newCollection!

blendWith: aCurve value: aNumber
	"blend self with aCurve, 0 means self and 1 means aCurve"

	(self pressure notNil or: [ aCurve pressure notNil or: [
	self saturation notNil or: [ aCurve saturation notNil or: [
	self interpolation ~= aCurve interpolation]]]])
		ifTrue: [self unblendable].

	^(self class
		controlPoints: (self controlPoints blendWith: aCurve controlPoints value: aNumber)
		interpolation: self interpolation "it doesn't matter which"
		cyclic: (self cyclic blendWith: aCurve cyclic value: aNumber))
		controlPointVariation: (self controlPointVariation blendWith: aCurve controlPointVariation value: aNumber);
		color: (self color blendWith: aCurve color value: aNumber)! !

!Curve methodsFor: 'copying'!

postCopy
	"Do post-copy stuff."

	super postCopy.
	controlPoints := (controlPoints collect: [ :p | p copy]).
	pressure := pressure copy.
	saturation := saturation copy.
	dirty := true.
	estimateLength := nil.
	collectionIndex := nil.
	arrayLength := nil.
	arrayX := nil.
	arrayY := nil.
	arrayWeight := nil.
	arrayPressure := nil.
	arraySaturation := nil.! !

!Curve methodsFor: 'transforming'!

scaleBy: amount 
	"Answer the receiver scaled by the argument amount, which can be
	a Point or a scalar value."

	self verticesDo: [ :c | c scaleBy: amount].
	self markDirty.!

scaledBy: amount 
	"Answer a new object scaled by the argument amount, which can be
	a Point or a scalar value."

	^self copy scaleBy: amount!

translateBy: amount 
	"Answer the receiver translated by amount, which can be
	a Point or a scalar value."

	self verticesDo: [ :c | c translateBy: amount].
	self markDirty.!

translatedBy: amount 
	"Answer a new object translated by amount, which can be
	a Point or a scalar value."

	^self copy translateBy: amount! !

!Curve methodsFor: 'printing'!

printOn: aStream

	aStream print: self class; space; nextPut: ${; cr.
	self saveTxtOn: aStream.
	aStream nextPut: $}; cr.! !

!Curve methodsFor: 'save txt / print eps'!

saveTxtOn: aStream
	"save self in a special TXT-format"

	aStream
		tab; nextPutAll: 'points: {'; cr.
	self verticesDo: [ :p | 
		aStream tab; tab. p saveTxtOn: aStream. aStream cr.
	].
	aStream
		tab; nextPut: $}; 
		nextPutAll: ' interpolation: ';
			nextPutAll: self interpolation asString.
	self cyclic ifTrue: [aStream nextPutAll: ' cyclic: true'].
	aStream cr.
	self pressure notNil ifTrue: [
		aStream tab; nextPutAll: 'pressure: '.
		self pressure saveTxtOn: aStream.
		aStream cr.
	].
	self saturation notNil ifTrue: [
		aStream tab; nextPutAll: 'saturation: '.
		self saturation saveTxtOn: aStream.
		aStream cr.
	].
	self controlPointVariation = 0.0 ifFalse: [
		aStream tab; nextPutAll: 'controlPointVariation: '.
		self controlPointVariation saveTxtOn: aStream.
		aStream cr.
	].
	self color = ColorValue black ifFalse: [
		aStream tab; nextPutAll: 'color: '.
		self color saveTxtOn: aStream.
		aStream cr.
	].! !

!Curve methodsFor: 'private'!

condenseControlPoints: epsilon

	| newControlPoints pOld condenseDirty |

	condenseDirty := false.
	newControlPoints := OrderedCollection new: self controlPoints size.
	pOld := 1e16@1e16. "jwd"
	self verticesDo: [ :pNew |
		(pOld dist: pNew) < epsilon
			ifTrue: [condenseDirty := true]
			ifFalse: [newControlPoints add: pNew. pOld := pNew].
	].

	condenseDirty ifTrue: [self controlPoints: newControlPoints].!

indexOf: u

	u < 0.0 ifTrue: [^1].
	^(collectionIndex indexForInserting: u) - 1.!

tLeft: tl xLeft: xl tRight: tr xRight: xr flatness2: flatness2 toOrderedCollection: oc

	| tm xm |

	tm := tl + tr / 2.0.
	xm := self pointAt: tm.

	((xl dist: xm) > flatness2 or: [(xm dist: xr) > flatness2])
			ifTrue: [
				self tLeft: tl xLeft: xl tRight: tm xRight: xm flatness2: flatness2 toOrderedCollection: oc.
				self tLeft: tm xLeft: xm tRight: tr xRight: xr flatness2: flatness2 toOrderedCollection: oc
			]
			ifFalse: [
				oc add: xm.
				oc add: xr
			].!

tOf: u index: index

	index = collectionIndex size ifTrue: [ "last segment"
		^u - (collectionIndex at: index) / (1.0 - (collectionIndex at: index))
	].
	^u - (collectionIndex at: index) / ((collectionIndex at: index +1) - (collectionIndex at: index))! !

!Curve methodsFor: 'enumerating'!

verticesDo: aBlock
	"Evaluate the supplied block for every vertex in the receiver."

	self controlPoints do: aBlock! !

!Curve class methodsFor: 'instance creation'!

controlPoints: listOfPoints

	^self controlPoints: listOfPoints interpolation: #chordal!

controlPoints: listOfPoints interpolation: interpolationProperty

	^self controlPoints: listOfPoints interpolation: interpolationProperty cyclic: false!

controlPoints: listOfPoints interpolation: interpolationProperty cyclic: isCyclic 

	listOfPoints size > 1 ifTrue: [^self new
			controlPoints: listOfPoints
			interpolation: interpolationProperty
			cyclic: isCyclic].
	^nil!

new

	^super new initialize!

points: listOfPoints

	^self controlPoints: (listOfPoints collect: [ :p | ControlPoint coordinates: p])! !

Interpolation subclass: #Interpolation2D
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interpolation'!

!BlockClosure methodsFor: 'polynomial functions'!

pointAt: t
	"behave like the 'polynomial functions'-protocol
	see Number, Point, Vector3 and Vector4 too"

	^self value: t! !

Object subclass: #StyleCatalog
	instanceVariableNames: 'style outputParameters catalog styleCache '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'StyleCatalog'!

!StyleCatalog methodsFor: 'initialize-release'!

initialize

	self outputParameters: (OutputParameters device: CurvePreferences defaultDevice).
	self catalog: Dictionary new. "the styleCache will be cleared"! !

!StyleCatalog methodsFor: 'accessing'!

catalog

	^catalog!

catalog: aDictionary

	catalog := aDictionary!

outputParameters

	^outputParameters!

outputParameters: aOutputParameters

	outputParameters := aOutputParameters!

style

	^style!

style: aStyle

	style := aStyle! !

!StyleCatalog methodsFor: 'cache'!

activateStyle: aPath

	self loadStyle: (self getFileNamePackage: aPath package type: aPath type)!

clearStyleCache

	styleCache := Dictionary new.
	self loadStyle: #default. "one style must exist"!

getFileNamePackage: package type: type

	(package isNil or: [type isNil]) ifTrue: [^#default].
	^((self catalog at: package asSymbol ifAbsent: [^#default])
		at: type asSymbol ifAbsent: [^#default])!

loadStyle: aStringOrSymbol

	| name |

	name := Style defaultTxtPath, aStringOrSymbol, '.crv'.
	self style: (styleCache at: name ifAbsentPut: [Style loadTxt: name]).
	self style outputParameters: self outputParameters.! !

!StyleCatalog methodsFor: 'diplaying'!

displayCurve: aPath on: aGraphicsContext

	self activateStyle: aPath.
	self style displayCurve: aPath on: aGraphicsContext!

displayPath: aPath on: aGraphicsContext

	self activateStyle: aPath.
	self style displayPath: aPath on: aGraphicsContext.! !

!StyleCatalog methodsFor: 'printing'!

printOn: aStream

	aStream print: self class; cr; cr.
	self saveTxtOn: aStream! !

!StyleCatalog methodsFor: 'save txt / print eps'!

saveTxtOn: aStream
	"save self in a special TXT-format"

	self catalog keysAndValuesDo: [ :packageName :package |
		aStream nextPut: $[; nextPutAll: packageName asString; nextPut: $]; cr.
		package keysAndValuesDo: [ :s :fileName |
			aStream nextPutAll: s asString; nextPut: $=; print: fileName asString; cr.
		].
		aStream cr.
	].! !

!StyleCatalog methodsFor: 'edit'!

closeAllWindows

	self dependents do: [ :d |
		(d isKindOf: View) ifTrue: [d topComponent controller closeAndUnschedule].
	].!

dontForgetEdit! !

!StyleCatalog class methodsFor: 'instance creation'!

catalog: aDictionary

	^(self new catalog: aDictionary) clearStyleCache!

new

	^super new initialize! !

!StyleCatalog class methodsFor: 'load txt'!

loadTxtOn: aStream
	"load from a special TXT-format"

	^(StyleCatalogParser parse: aStream nextChunk as: #catalog) at: 1! !

!StyleCatalog class methodsFor: 'examples'!

example
	"StyleCatalog example"

	^(self loadTxt: self defaultTxtPath, 'test.cat') inspect! !

Object subclass: #LogicAttributes
	instanceVariableNames: 'value '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LogicAttributes'!

!LogicAttributes methodsFor: 'accessing'!

value

	^value!

value: aFloat

	value := aFloat! !

!LogicAttributes methodsFor: 'comparing'!

= anObject

	^value = anObject value!

hash
	"Answer a SmallInteger that is equal to the hash value of 
	any object that is equal to (=) the receiver. 
	When two objects are not =, their hash values may or may not be the same.
	The hash value may change based on the receiver's contents."

	^value hash! !

!LogicAttributes methodsFor: 'printing'!

printOn: aStream

	^self storeOn: aStream!

storeOn: aStream

	| const |

	(const := self class constantNameFor: self) isNil ifFalse: [
		aStream print: self class; space; nextPutAll: const asString.
		^self
	].

	aStream print: self value! !

!LogicAttributes methodsFor: 'save txt / print eps'!

saveTxtOn: aStream
	"save self in a special TXT-format"

	| const |

	(const := self class constantNameFor: self) isNil ifFalse: [
		aStream nextPutAll: const asString.
		^self
	].

	aStream print: self value! !

!LogicAttributes class methodsFor: 'instance creation'!

value: aFloat

	^self new value: aFloat! !

!LogicAttributes class methodsFor: 'constants'!

zero

	^self value: 0.0! !

!LogicAttributes class methodsFor: 'private'!

constantNameFor: instance
	" If instance is a constant that can be
	obtained by sending a known constant
	message to this class, answer the message
	selector, otherwise answer nil. "

	(self class organization listAtCategoryNamed: #constants) do: [:const |
		(self perform: const) = instance ifTrue: [^const]
	].
	^nil! !

LogicAttributes subclass: #LogicStrokeLength
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LogicAttributes'!

!LogicStrokeLength class methodsFor: 'constants'!

long

	^self value: 50.0!

normal

	^self value: 30.0!

short

	^self value: 20.0!

veryLong

	^self value: 70.0!

veryShort

	^self value: 10.0! !

Object subclass: #InterpolationPolynomial
	instanceVariableNames: 'polynomial '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interpolation'!

!InterpolationPolynomial methodsFor: 'initialize-release'!

constant: aValue

	polynomial := aValue!

kubic: aVector4

	polynomial := (Matrix4x4 lagrange * aVector4) polynomialSimplify!

kubicT0: aValue0 t1_3: aValue1_3 t2_3: aValue2_3 t1: aValue1

	^self kubic: (Vector4 x: aValue0 y: aValue1_3 z: aValue2_3 w: aValue1)!

linear: aPoint

	^self linearT0: aPoint x t1: aPoint y!

linearT0: aValue0 t1: aValue1

	polynomial := (Point x: aValue1 - aValue0 y: aValue0) polynomialSimplify!

quadratic: aVector3

	polynomial := (Matrix3x3 lagrange * aVector3) polynomialSimplify!

quadraticT0: aValue0 t1_2: aValue1_2 t1: aValue1

	^self quadratic: (Vector3 x: aValue0 y: aValue1_2 z: aValue1)!

withArray: aArray

	| size |

	size := aArray size.
	(size < 1 or: [size > 4 ]) ifTrue: [^self error: 'array sizes do not match'].
	size = 1 ifTrue: [^self constant: (aArray at: 1)].
	size = 2 ifTrue: [^self linearT0: (aArray at: 1) t1: (aArray at: 2)].
	size = 3 ifTrue: [^self quadraticT0: (aArray at: 1) t1_2: (aArray at: 2) t1: (aArray at: 3)].
	size = 4 ifTrue: [^self kubicT0: (aArray at: 1) t1_3: (aArray at: 2) t2_3: (aArray at: 3) t1: (aArray at: 4)].! !

!InterpolationPolynomial methodsFor: 'accessing'!

polynomial: aPolynomial
	"aPolynomial must be an instance of a class, wich understands:
	pointAt:, derivativeAt:, secondDerivativeAt:, thirdDerivativeAt:, polynomialSimplify"

	polynomial := aPolynomial! !

!InterpolationPolynomial methodsFor: 'polynomial functions'!

derivativeAt: t

	^polynomial derivativeAt: t!

hornerSchemeAt: t

	^Vector4 new
		x: (polynomial pointAt: t)
		y: (polynomial derivativeAt: t)
		z: (polynomial secondDerivativeAt: t)
		w: (polynomial thirdDerivativeAt: t)!

pointAt: t

	^polynomial pointAt: t!

polynomialSimplify

	polynomial := polynomial polynomialSimplify!

secondDerivativeAt: t

	^polynomial secondDerivativeAt: t!

thirdDerivativeAt: t

	^polynomial thirdDerivativeAt: t!

value: t
	"behavior like a BlockClosure"
	^polynomial pointAt: t! !

!InterpolationPolynomial class methodsFor: 'instance creation'!

constant: aValue

	^self new constant: aValue!

kubic: aVector4

	^self new kubic: aVector4!

kubicT0: aValue0 t1_3: aValue1_3 t2_3: aValue2_3 t1: aValue1

	^self kubic: (Vector4 x: aValue0 y: aValue1_3 z: aValue2_3 w: aValue1)!

linear: aPoint

	^self new linearT0: aPoint x t1: aPoint y!

linearT0: aValue0 t1: aValue1

	^self new linearT0: aValue0 t1: aValue1!

quadratic: aVector3

	^self new quadratic: aVector3!

quadraticT0: aValue0 t1_2: aValue1_2 t1: aValue1

	^self quadratic: (Vector3 x: aValue0 y: aValue1_2 z: aValue1)!

withArray: aArray

	^self new withArray: aArray! !

Object subclass: #CurvePreferences
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CurveBasics'!

CurvePreferences class
	instanceVariableNames: 'txtPath txtName '!

!CurvePreferences class methodsFor: 'class initialization'!

initialize
	"CurvePreferences initialize"

	txtPath := Dictionary new.
	txtPath add: #Style -> 'CrvStyle\'.
	txtPath add: #StyleCatalog -> 'Catalog\'.
	txtPath add: #CurvePainter -> 'Crv\'.

	txtName := Dictionary new.
	txtName add: #Style -> '*.crv'.
	txtName add: #StyleCatalog -> '*.cat'.
	txtName add: #CurvePainter -> '*.crv'.! !

!CurvePreferences class methodsFor: 'path / name'!

defaultEpsName

	^'*.eps'!

defaultEpsPath

	^'EPS\'!

defaultTxtName: aClass

	^txtName at: aClass name ifAbsent: ['*.*']!

defaultTxtPath: aClass

	^txtPath at: aClass name ifAbsent: ['']! !

!CurvePreferences class methodsFor: 'color'!

backgroundColor

	^ColorValue white! !

!CurvePreferences class methodsFor: 'output parameters'!

defaultDevice

	^#Screen! !

Object subclass: #OutputParameters
	instanceVariableNames: 'quality qualityLength smallestLength lineOptimization width1 stroke1 shorten1 '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CurveBasics'!
OutputParameters comment:
'I store all device dependent parameters for the visualisation of a style, a path, or a superposition.

Instance Variables:
	quality			Integer	"current quality (1=draft 2=normal 3=presentation)"
	qualityLength	Array	"draft normal presentation"
	smallestLength	Float	"points nearer than this value will be condensed"
	lineOptimization	Float	"0.0 means no optimize"
	width1			Float	"what means pressure = 1.0"
	stroke1			Float	"what means strokeLength = 1.0"
	shorten1		Float	"what means shorten(Start End) = 1.0"
'!

!OutputParameters methodsFor: 'accessing'!

lineOptimization

	^lineOptimization!

lineOptimization: aNumber

	lineOptimization := aNumber!

maxLength

	^qualityLength at: quality!

quality

	^quality!

quality: aNumber

	quality := aNumber!

qualityLength

	^qualityLength!

shorten1

	^shorten1!

shorten1: aNumber

	shorten1 := aNumber!

smallestLength

	^smallestLength!

smallestLength: aNumber

	smallestLength := aNumber!

stroke1

	^stroke1!

stroke1: aNumber

	stroke1 := aNumber!

width1

	^width1!

width1: aNumber

	width1 := aNumber! !

!OutputParameters methodsFor: 'printing'!

printOn: aStream

	self saveTxtOn: aStream! !

!OutputParameters methodsFor: 'save txt / print eps'!

saveTxtOn: aStream
	"save self in a special TXT-format"

	aStream
		print: self class; cr;
		tab; nextPutAll: 'quality: '; nextPutAll: (self class qualityArray at: quality) asString; cr;
		tab; nextPutAll: 'qualityLength: '; print: qualityLength; cr;
		tab; nextPutAll: 'smallestLength: '; print: smallestLength; cr;
		tab; nextPutAll: 'lineOptimization: '; print: lineOptimization; cr;
		tab; nextPutAll: 'width1: '; print: width1; cr;
		tab; nextPutAll: 'stroke1: '; print: stroke1; cr;
		tab; nextPutAll: 'shorten1: '; print: shorten1; cr! !

!OutputParameters methodsFor: 'private'!

device: aSymbol

	self perform: ('OP',aSymbol) asSymbol!

OPAnimate

	quality := self class qualityNormal.
	qualityLength := #(10.0 1.0 0.25) copy. "draft normal presentation"
	smallestLength := 1.0. "points nearer than this value will be condensed"
	lineOptimization := 1.0. "0.0 means no optimize"
	width1 := 1.0. "what means pressure = 1.0"
	stroke1 := 1.0. "what means strokeLength = 1.0"
	shorten1 :=1.0. "what means shorten(Start End) = 1.0"!

OPEps600Dpi

	quality := self class qualityNormal.
	qualityLength := #(10.0 1.0 0.25) copy. "draft normal presentation"
	smallestLength := 1.0. "points nearer than this value will be condensed"
	lineOptimization := 1.0. "0.0 means no optimize"
	width1 := 1.0. "what means pressure = 1.0"
	stroke1 := 1.0. "what means strokeLength = 1.0"
	shorten1 := 1.0. "what means shorten(Start End) = 1.0"!

OPNormalized

	quality := self class qualityNormal.
	qualityLength := #(0.2 0.02 0.005) copy. "draft normal presentation"
	smallestLength := 0.005. "points nearer than this value will be condensed"
	lineOptimization := 0.005. "0.0 means no optimize"
	width1 := 0.005. "what means pressure = 1.0"
	stroke1 := 0.005. "what means strokeLength = 1.0"
	shorten1 := 0.005. "what means shorten(Start End) = 1.0"!

OPScreen

	quality := self class qualityNormal.
	qualityLength := #(40.0 4.0 1.0) copy. "draft normal presentation"
	"qualityLength := #(1.0 1.0 1.0) copy." "draft normal presentation"
	smallestLength := 1.0. "points nearer than this value will be condensed"
	lineOptimization := 1.0. "0.0 means no optimize"
	width1 := 1.0. "what means pressure = 1.0"
	stroke1 := 1.0. "what means strokeLength = 1.0"
	shorten1 :=1.0. "what means shorten(Start End) = 1.0"! !

!OutputParameters class methodsFor: 'instance creation'!

device: aSymbol

	^self new device: aSymbol! !

!OutputParameters class methodsFor: 'constants'!

qualityArray

	^#(#qualityDraft #qualityNormal #qualityPresentation)!

qualityDraft

	^1!

qualityNormal

	^2!

qualityPresentation

	^3! !

Interpolation subclass: #Interpolation1D
	instanceVariableNames: 'values interpolation cyclic polynomials '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interpolation'!

!Interpolation1D methodsFor: 'initialize-release'!

keys: keyArray values: valueArray interpolation: interpolationProperty cyclic: isCyclic

	self cyclic: isCyclic.
	self values: valueArray.
	self interpolation: interpolationProperty.

	keyArray size = valueArray size ifTrue: [
		self keys: keyArray.
		self calcValue
	]
	ifFalse: [self error: 'collection sizes do not match']!

values: valueArray interpolation: interpolationProperty cyclic: isCyclic
	"a uniform interpolation"

	self cyclic: isCyclic.
	self values: valueArray.
	self interpolation: interpolationProperty.

	"value interpolation"
	self calcValue.

	self calcKeysUniform: polynomials size!

withArray: anArray interpolation: interpolationProperty cyclic: isCyclic

	| keyArray valueArray |
	
	keyArray := Array new: anArray size.
	valueArray := Array new: anArray size.
	anArray keysAndValuesDo: [ :i :p |
		keyArray at: i put: (p at: 1).
		valueArray at: i put: (p at: 2).
	].

	self
		keys: keyArray
		values: valueArray
		interpolation: interpolationProperty
		cyclic: isCyclic! !

!Interpolation1D methodsFor: 'accessing'!

cyclic

	^cyclic!

cyclic: aBoolean

	cyclic := aBoolean!

interpolation

	^interpolation!

interpolation: interpolationProperty

	interpolation := interpolationProperty!

polynomials

	^polynomials!

polynomials: anArray
	"Attention!!
	This method manipulate the attribute 'polynomials' directly.
	If you're not sure what exactly you do -
	please use 'values:interpolation:' or 'withArray:interpolation:' instead"

	polynomials := anArray.
	values := nil. "don't know the values"!

values

	^values!

values: valueArray

	values := valueArray! !

!Interpolation1D methodsFor: 'copying'!

postCopy
	"Do post-copy stuff."

	super postCopy.
	values := values copy.
	polynomials := polynomials copy.! !

!Interpolation1D methodsFor: 'polynomial functions'!

derivativeAt: u

	| index |
	
	index := self indexOf: u.
	^(polynomials at: index) derivativeAt: (self tOf: u index: index)!

hornerSchemeAt: u

	| index t polynomial |
	
	index := self indexOf: u.
	t := self tOf: u index: index.
	polynomial := polynomials at: index.
	^Vector4 new
		x: (polynomial pointAt: t)
		y: (polynomial derivativeAt: t)
		z: (polynomial secondDerivativeAt: t)
		w: (polynomial thirdDerivativeAt: t)!

pointAt: u

	| index |
	
	index := self indexOf: u.
	^(polynomials at: index) pointAt: (self tOf: u index: index)!

polynomialSimplify

	1 to: polynomials size do: [ :i |
		polynomials at: i put: ((polynomials at: i) polynomialSimplify)
	].!

secondDerivativeAt: u

	| index |
	
	index := self indexOf: u.
	^(polynomials at: index) secondDerivativeAt: (self tOf: u index: index)!

thirdDerivativeAt: u

	| index |
	
	index := self indexOf: u.
	^(polynomials at: index) thirdDerivativeAt: (self tOf: u index: index)! !

!Interpolation1D methodsFor: 'save txt / print eps'!

saveTxtOn: aStream
	"save self in a special TXT-format"

	aStream print: self class; cr.
	self areKeysUniform ifFalse: [
		aStream tab; nextPutAll: 'keys: {'.
		self keys do: [ :k | aStream print: k] separatedBy: [aStream space].
		self keys size < values size ifTrue: [aStream nextPutAll: ' 1.0'].
		aStream nextPut: $}; cr
	].
	aStream tab; nextPutAll: 'values: {'.
	self values do: [ :v | aStream print: v] separatedBy: [aStream space].
	aStream nextPut: $}; cr;
		tab; nextPutAll: 'interpolation: ';
		nextPutAll: (self class interpolationArray at: interpolation) asString.
	self cyclic ifTrue: [aStream cr; tab; nextPutAll: 'cyclic: true'].! !

!Interpolation1D methodsFor: 'private values'!

calcBSpline2

	self notSupportedYet
	"| p0 p1 p2 |

	polynomials := Array new: values size - 2.
	p0 := values at: 1.
	p1 := values at: 2.
	1 to: values size - 2 do: [ :i |
		p2 := values at: i + 2.
		polynomials at: i put: Matrix3x3 bSpline * (p0 @ p1 @ p2).
		p0 := p1. p1 := p2.
	]."!

calcBSpline3

	| p0 p1 p2 p3 matrix size newArray |

	matrix := Matrix4x4 bSpline.
	newArray := self prepareDoubleValues.
	size := newArray size - 3.
	polynomials := Array new: size.

	p0 := newArray at: 1.
	p1 := newArray at: 2.
	p2 := newArray at: 3.
	1 to: size do: [ :i |
		p3 := newArray at: i + 3.
		polynomials at: i put: matrix * (p0 @ p1 @ p2 @ p3).
		p0 := p1. p1 := p2. p2 := p3.
	].!

calcCatmull3

	| p0 p1 p2 p3 matrix size newArray |

	matrix := Matrix4x4 catmull.
	newArray := self prepareDoubleValues.
	size := newArray size - 3.
	polynomials := Array new: size.

	p0 := newArray at: 1.
	p1 := newArray at: 2.
	p2 := newArray at: 3.
	1 to: size do: [ :i |
		p3 := newArray at: i + 3.
		polynomials at: i put: matrix * (p0 @ p1 @ p2 @ p3).
		p0 := p1. p1 := p2. p2 := p3.
	].!

calcChordal3

	self calcCatmull3!

calcConstant0

	self notSupportedYet
	"polynomials := Array new: values size.
	1 to: values size do: [ :i | polynomials at: i put: (values at: i)]"!

calcLinear1

	| p0 p1 size newArray |

	newArray := self prepareDoubleValues.
	size := newArray size - 3.
	polynomials := Array new: size.
	p0 := newArray at: 2. "ignore the double"
	1 to: size do: [ :i |
		p1 := newArray at: i + 2.
		polynomials at: i put: (Point x: p1 - p0 y: p0).
		p0 := p1.
	].!

calcValue

	(interpolation = self class constant0) ifTrue: [self calcConstant0].
	(interpolation = self class linear1) ifTrue: [self calcLinear1].
	(interpolation = self class bSpline2) ifTrue: [self calcBSpline2].
	(interpolation = self class bSpline3) ifTrue: [self calcBSpline3].
	(interpolation = self class catmull3) ifTrue: [self calcCatmull3].
	(interpolation = self class chordal3) ifTrue: [self calcChordal3].
	self polynomialSimplify!

prepareDoubleValues
	"with double ends"
	| size newArray |

	size := values size + 2. "two extra"
	(self cyclic and: [values first ~= values last]) ifTrue: [size := size + 1].

	newArray := Array new: size.
	newArray replaceFrom: 2 to: 1 + values size with: values startingAt: 1.
	self cyclic ifTrue: [
		values first ~= values last ifTrue: [
			newArray at: size - 1 put: (newArray at: 2)
		].
		newArray at: 1 put: (newArray at: size - 2).
		newArray at: size put: (newArray at: 3).
	] ifFalse: [ "double start and end point"
		newArray at: 1 put: (newArray at: 2).
		newArray at: size put: (newArray at: size - 1).
	].

	^newArray! !

!Interpolation1D class methodsFor: 'instance creation'!

keys: keyArray values: valueArray interpolation: interpolationProperty

	^self new
		keys: keyArray
		values: valueArray
		interpolation: interpolationProperty
		cyclic: false!

keys: keyArray values: valueArray interpolation: interpolationProperty cyclic: isCyclic

	^self new
		keys: keyArray
		values: valueArray
		interpolation: interpolationProperty
		cyclic: isCyclic!

values: valueArray interpolation: interpolationProperty

	^self new
		values: valueArray
		interpolation: interpolationProperty
		cyclic: false!

values: valueArray interpolation: interpolationProperty cyclic: isCyclic

	^self new
		values: valueArray
		interpolation: interpolationProperty
		cyclic: isCyclic!

withArray: anArray interpolation: interpolationProperty

	^self new
		withArray: anArray
		interpolation: interpolationProperty
		cyclic: false!

withArray: anArray interpolation: interpolationProperty cyclic: isCyclic

	^self new
		withArray: anArray
		interpolation: interpolationProperty
		cyclic: isCyclic! !

Curve subclass: #Path
	instanceVariableNames: 'package type shortenStart shortenEnd '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path'!

!Path methodsFor: 'initialize-release'!

initialize

	super initialize.
	self package: nil.
	self type: nil.
	self shortenStart: 0.0.
	self shortenEnd: 0.0.! !

!Path methodsFor: 'accessing'!

package

	^package!

package: aSymbol

	package := aSymbol!

shortenEnd

	^shortenEnd!

shortenEnd: aNumber

	shortenEnd := aNumber!

shortenStart

	^shortenStart!

shortenStart: aNumber

	shortenStart := aNumber!

type

	^type!

type: aSymbol

	type := aSymbol! !

!Path methodsFor: 'save txt / print eps'!

saveTxtOn: aStream
	"save self in a special TXT-format"

	super saveTxtOn: aStream.
	self package notNil ifTrue: [
		aStream tab; nextPutAll: 'package: '; nextPutAll: self package asSymbol; cr.
	].
	self type notNil ifTrue: [
		aStream tab; nextPutAll: 'type: '; nextPutAll: self type asSymbol; cr.
	].
	self shortenStart = 0.0 ifFalse: [
		aStream tab; nextPutAll: 'shortenStart: '; nextPutAll: self shortenStart asString; cr.
	].
	self shortenEnd = 0.0 ifFalse: [
		aStream tab; nextPutAll: 'shortenEnd: '; nextPutAll: self shortenEnd asString; cr.
	].! !

StyleCatalogParser initialize!

StyleParser initialize!

CurvePreferences initialize!


